@@ -1,21 +1,24 @@
 /****************************************************************************
-** $Id: qmsgbox.h,v 2.0 1996/09/24 10:15:42 agulbra Exp $
+** $Id: qmsgbox.h,v 2.9 1997/03/23 17:51:15 hanord Exp $
 **
 ** Definition of QMessageBox class
 **
 ** Created : 950503
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
 #ifndef QMSGBOX_H
+#define QMSGBOX_H
 
 #include "qdialog.h"
 
@@ -31,6 +34,11 @@ public:
 			const char *text,  const char *buttonText=0,
 			QWidget *parent=0, const char *name=0 );
 
+    static bool query( const char *caption,
+		       const char *text,  const char *yesButtonText=0,
+		       const char *noButtonText=0,
+		       QWidget *parent=0, const char *name=0 );
+
     QMessageBox( QWidget *parent=0, const char *name=0 );
 
     const char *text()		const;
@@ -50,6 +58,8 @@ private:
     void	*reserved1;
     void	*reserved2;
 
+    QPushButton *button2() { return (QPushButton*) reserved1; }
+
 private:	// Disabled copy constructor and operator=
     QMessageBox( const QMessageBox & ) {}
     QMessageBox &operator=( const QMessageBox & ) { return *this; }
