@@ -1,17 +1,19 @@
 /****************************************************************************
-** $Id: qaccel.cpp,v 2.0 1996/09/24 10:15:43 agulbra Exp $
+** $Id: qaccel.cpp,v 2.2 1996/10/07 08:23:18 hanord Exp $
 **
 ** Implementation of QAccel class
 **
 ** Created : 950419
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -22,7 +24,7 @@
 #include "qlist.h"
 #include "qsignal.h"
 
-RCSTAG("$Id: qaccel.cpp,v 2.0 1996/09/24 10:15:43 agulbra Exp $");
+RCSTAG("$Id: qaccel.cpp,v 2.2 1996/10/07 08:23:18 hanord Exp $");
 
 
 /*!
@@ -65,7 +67,7 @@ struct QAccelItem {				// internal accelerator item
     QSignal    *signal;
 };
 
-typedef declare(QListM,QAccelItem) QAccelList;	// internal accelerator list
+typedef Q_DECLARE(QListM,QAccelItem) QAccelList; // internal accelerator list
 
 
 static QAccelItem *find_id( QAccelList *list, int id )
