@@ -1,24 +1,26 @@
 /****************************************************************************
-** $Id: qcolor.cpp,v 2.0 1996/09/24 10:15:48 agulbra Exp $
+** $Id: qcolor.cpp,v 2.3 1997/03/10 12:03:20 hanord Exp $
 **
 ** Implementation of QColor class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
 #include "qcolor.h"
 #include "qdstream.h"
 
-RCSTAG("$Id: qcolor.cpp,v 2.0 1996/09/24 10:15:48 agulbra Exp $");
+RCSTAG("$Id: qcolor.cpp,v 2.3 1997/03/10 12:03:20 hanord Exp $");
 
 
 /*!
@@ -62,7 +64,7 @@ RCSTAG("$Id: qcolor.cpp,v 2.0 1996/09/24 10:15:48 agulbra Exp $");
   color and save it in the internal table.
   </ol>
 
-  \sa QPalette, QColorGroup
+  \sa QPalette, QColorGroup, QApplication::setColorSpec()
 */
 
 /*****************************************************************************
@@ -70,8 +72,8 @@ RCSTAG("$Id: qcolor.cpp,v 2.0 1996/09/24 10:15:48 agulbra Exp $");
  *****************************************************************************/
 
 #if defined(_WS_WIN_)
-#define COLOR0_PIX 0
-#define COLOR1_PIX 0x00ffffff
+#define COLOR0_PIX 0x00ffffff
+#define COLOR1_PIX 0
 #else
 #define COLOR0_PIX 0
 #define COLOR1_PIX 1
@@ -82,7 +84,7 @@ const QColor color1	( 0x00000000, COLOR1_PIX );
 const QColor black	(   0,	 0,   0 );
 const QColor white	( 255, 255, 255 );
 const QColor darkGray	( 128, 128, 128 );
-const QColor gray	( 160, 160, 160 );
+const QColor gray	( 160, 160, 164 );
 const QColor lightGray	( 192, 192, 192 );
 const QColor red	( 255,	 0,   0 );
 const QColor green	(   0, 255,   0 );
@@ -131,7 +133,7 @@ void QColor::initglobals()
     ((QColor*)(&::black))	->setRgb(   0,	 0,   0 );
     ((QColor*)(&::white))	->setRgb( 255, 255, 255 );
     ((QColor*)(&::darkGray))	->setRgb( 128, 128, 128 );
-    ((QColor*)(&::gray))	->setRgb( 160, 160, 160 );
+    ((QColor*)(&::gray))	->setRgb( 160, 160, 164 );
     ((QColor*)(&::lightGray))	->setRgb( 192, 192, 192 );
     ((QColor*)(&::red))		->setRgb( 255,	 0,   0 );
     ((QColor*)(&::green))	->setRgb(   0, 255,   0 );
@@ -278,9 +280,9 @@ void QColor::hsv( int *h, int *s, int *v ) const
     int delta = max-min;
     *v = max;					// calc value
     *s = max ? (510*delta+max)/(2*max) : 0;
-    if ( *s == 0 )
+    if ( *s == 0 ) {
 	*h = -1;				// undefined hue
-    else {
+    } else {
 	switch ( whatmax ) {
 	    case 0:				// red is max component
 		if ( g >= b )
@@ -323,9 +325,9 @@ void QColor::setHsv( int h, int s, int v )
     }
 #endif
     int r=v, g=v, b=v;
-    if ( s == 0 || h == -1 )			// achromatic case
+    if ( s == 0 || h == -1 ) {			// achromatic case
 	;
-    else {					// chromatic case
+    } else {					// chromatic case
 	if ( (uint)h >= 360 )
 	    h %= 360;
 	uint f = h%60;
@@ -507,7 +509,7 @@ QColor QColor::dark( int factor ) const
 
   Lazy color allocation is enabled by default.
 
-  \sa lazyAlloc()
+  \sa lazyAlloc(), pixel(), alloc()
 */
 
 void QColor::setLazyAlloc( bool enable )
@@ -522,6 +524,8 @@ void QColor::setLazyAlloc( bool enable )
 
   This value is used by the underlying window system to refer to a color.
   It can be thought of as an index into the display hardware's color table.
+
+  \sa setLazyAlloc(), alloc()
 */
 
 
