@@ -1,35 +1,37 @@
 /****************************************************************************
-** $Id: qfnt_x11.cpp,v 2.0 1996/09/24 10:15:51 agulbra Exp $
+** $Id: qfnt_x11.cpp,v 2.14 1997/04/04 15:54:13 hanord Exp $
 **
 ** Implementation of QFont, QFontMetrics and QFontInfo classes for X11
 **
 ** Created : 940515
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
 #include "qwidget.h"
 #include "qpainter.h"
+#include "qfontdta.h"
 #include "qcache.h"
 #include "qdict.h"
 #include <ctype.h>
+#include <stdio.h>
 #include <stdlib.h>
 #define GC GC_QQQ
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
 #include <X11/Xatom.h>
-#define QXFontStruct XFontStruct
-#include "qfontdta.h"
 
-RCSTAG("$Id: qfnt_x11.cpp,v 2.0 1996/09/24 10:15:51 agulbra Exp $");
+RCSTAG("$Id: qfnt_x11.cpp,v 2.14 1997/04/04 15:54:13 hanord Exp $");
 
 
 static const int fontFields = 14;
@@ -53,15 +55,12 @@ enum FontFieldNames {				// X LFD fields
 
 // Internal functions
 
-static QXFontData *loadXFont	   ( const QString &fontName, bool *newFont );
-static bool	   parseXFontName  ( QString &fontName, char **tokens );
-static QString	   bestFitFamily   ( const QString & );
-static char	 **getXFontNames   ( const char *pattern, int *count );
-static bool	   smoothlyScalable( const char *fontName );
-static bool	   fontExists	   ( const char *fontName );
-static int	   computeLineWidth( const char *fontName );
-static int	   getWeight	   ( const char *weightString,
-				     bool adjustScore = FALSE );
+static bool	parseXFontName( QString &fontName, char **tokens );
+static char   **getXFontNames( const char *pattern, int *count );
+static bool	smoothlyScalable( const char *fontName );
+static bool	fontExists( const char *fontName );
+static int	computeLineWidth( const char *fontName );
+static int	getWeight( const char *weightString, bool adjustScore=FALSE );
 
 
 // QFont_Private accesses QFont protected functions
@@ -69,11 +68,12 @@ static int	   getWeight	   ( const char *weightString,
 class QFont_Private : public QFont
 {
 public:
-    int	    fontMatchScore( char  *fontName,	  QString &buffer,
-			    float *pointSizeDiff, int	  *weightDiff,
-			    bool  *scalable	, bool	  *polymorphic );
-    QString bestMatch( const QString &pattern, int *score );
-    QString bestFamilyMember( const QString &family, int *score );
+    int	    fontMatchScore( char *fontName, QString &buffer,
+			    float *pointSizeDiff, int *weightDiff,
+			    bool *scalable, bool *polymorphic,
+			    int *resx, int *resy );
+    QString bestMatch( const char *pattern, int *score );
+    QString bestFamilyMember( const char *family, int *score );
     QString findFont( bool *exact );
 };
 
@@ -81,51 +81,113 @@ public:
 
 
 /*****************************************************************************
-  QFont cache to make font loading faster than using XLoadQueryFont
+  QFontInternal contains X11 font data: an XLFD font name ("-*-*-..") and
+  an X font struct.
+
+  Two global dictionaries and a cache hold QFontInternal objects, which
+  are shared between all QFonts.
+  This mechanism makes font loading much faster than using XLoadQueryFont.
  *****************************************************************************/
 
-struct QXFontData {
-    QString	  name;
-    QXFontStruct *f;
-    bool dirty() { return f == 0; }
-    QXFontData() {}
-   ~QXFontData() { if( f ) XFreeFont( qt_xdisplay(), f ); }
+class QFontInternal
+{
+public:
+   ~QFontInternal();
+    bool	    dirty() const;
+    const char	   *name()  const;
+    XFontStruct	   *fontStruct() const;
+    const QFontDef *spec()  const;
+    int		    lineWidth() const;
+    void	    reset();
+private:
+    QFontInternal( const QString & );
+    QString	    n;
+    XFontStruct	   *f;
+    QFontDef	    s;
+    int		    lw;
+    friend void QFont::load(HANDLE) const;
+    friend void QFont::initFontInfo() const;
 };
 
+inline QFontInternal::QFontInternal( const QString &name )
+    : n(name), f(0)
+{
+    s.dirty = TRUE;
+}
+
+inline bool QFontInternal::dirty() const
+{
+    return f == 0;
+}
+
+inline const char *QFontInternal::name() const
+{
+    return n;
+}
+
+inline XFontStruct *QFontInternal::fontStruct() const
+{
+    return f;
+}
+
+inline const QFontDef *QFontInternal::spec() const
+{
+    return &s;
+}
+
+inline int QFontInternal::lineWidth() const
+{
+    return lw;
+}
+
+inline void QFontInternal::reset()
+{
+    if ( f ) {
+	XFreeFont( QPaintDevice::x__Display(), f );
+	f = 0;
+    }
+}
+
+inline QFontInternal::~QFontInternal()
+{
+    reset();
+}
+
 
 static const int reserveCost   = 1024*100;
 static const int fontCacheSize = 1024*1024*4;
 
 
-declare(QCacheM,QXFontData);			// inherited by QFontCache
-typedef declare(QCacheIteratorM,QXFontData) QFontCacheIt;
-typedef declare(QDictM,QXFontData)	    QFontDict;
-typedef declare(QDictIteratorM,QXFontData)  QFontDictIt;
+Q_DECLARE(QCacheM,QFontInternal);		// inherited by QFontCache
+typedef Q_DECLARE(QCacheIteratorM,QFontInternal) QFontCacheIt;
+typedef Q_DECLARE(QDictM,QFontInternal)		 QFontDict;
+typedef Q_DECLARE(QDictIteratorM,QFontInternal)  QFontDictIt;
 
 
-class QFontCache : public QCacheM(QXFontData)
+class QFontCache : public QCacheM(QFontInternal)
 {
 public:
     QFontCache( int maxCost, int size=17, bool cs=TRUE, bool ck=TRUE )
-	: QCacheM(QXFontData)(maxCost,size,cs,ck) {}
-    void deleteItem( GCI d );
+	: QCacheM(QFontInternal)(maxCost,size,cs,ck) {}
+    void deleteItem( GCI );
 };
 
 void QFontCache::deleteItem( GCI d )
 {
-    QXFontData *xfd = (QXFontData *)d;
-    if ( !xfd->dirty() )
-	XFreeFont( qt_xdisplay(), xfd->f );
-    xfd->f = 0;
+    QFontInternal *fin = (QFontInternal *)d;
+    fin->reset();
 }
 
 
-struct QXFontName {
+struct QXFontName
+{
+    QXFontName( const QString &n, bool e ) : name(n), exactMatch(e) {}
     QString name;
     bool    exactMatch;
 };
 
-typedef declare(QDictM,QXFontName) QFontNameDict;
+typedef Q_DECLARE(QDictM,QXFontName) QFontNameDict;
+
 
 static QFontCache    *fontCache	     = 0;	// cache of loaded fonts
 static QFontDict     *fontDict	     = 0;	// dict of all loaded fonts
@@ -133,41 +195,6 @@ static QFontNameDict *fontNameDict   = 0;	// dict of matched font names
 QFont		     *QFont::defFont = 0;	// default font
 
 
-/*****************************************************************************
-  QFontData member functions
- *****************************************************************************/
-
-QFontData::QFontData()
-{
-    xfd = 0;
-}
-
-QFontData::~QFontData()
-{
-  // Font data is cleaned up by font cache and font dict
-}
-
-QFontData::QFontData( const QFontData &d )
-{
-    req = d.req;
-    act = d.act;
-    exactMatch = d.exactMatch;
-    lineW = d.lineW;
-    xfd = d.xfd;				// safe to copy
-}
-
-
-QFontData &QFontData::operator=( const QFontData &d )
-{
-    req = d.req;
-    act = d.act;
-    exactMatch = d.exactMatch;
-    lineW = d.lineW;
-    xfd = d.xfd;				// safe to copy
-    return *this;
-}
-
-
 //
 // This function returns the X font struct for a QFontData.
 // It is called from QPainter::drawText().
@@ -175,7 +202,7 @@ QFontData &QFontData::operator=( const QFontData &d )
 
 XFontStruct *qt_get_xfontstruct( QFontData *d )
 {
-    return d->xfd->f;
+    return d->fin->fontStruct();
 }
 
 
@@ -185,15 +212,21 @@ XFontStruct *qt_get_xfontstruct( QFontData *d )
 
 /*!
   Internal function that initializes the font system.
+
+  \internal
+  The font cache and font dict do not alloc the keys. The key is a QString
+  which is shared between QFontInternal and QXFontName.
 */
 
-void QFont::initialize()			// called when starting up
+void QFont::initialize()
 {
-    fontCache = new QFontCache( fontCacheSize );// create font cache
+    if ( fontCache )
+	return;
+    fontCache = new QFontCache( fontCacheSize, 29, TRUE, FALSE );
     CHECK_PTR( fontCache );
-    fontDict  = new QFontDict( 29 );		// create font dict
+    fontDict  = new QFontDict( 29, TRUE, FALSE );
     CHECK_PTR( fontDict );
-    fontNameDict = new QFontNameDict( 29 );	// create font name dictionary
+    fontNameDict = new QFontNameDict( 29, TRUE, TRUE );
     CHECK_PTR( fontNameDict );
     fontNameDict->setAutoDelete( TRUE );
     if ( !defFont )
@@ -204,12 +237,14 @@ void QFont::initialize()			// called when starting up
   Internal function that cleans up the font system.
 */
 
-void QFont::cleanup()				// called when terminating app
+void QFont::cleanup()
 {
-    fontDict->setAutoDelete( TRUE );
     delete defFont;
+    defFont = 0;
+    delete fontCache;
+    fontCache = 0;
+    fontDict->setAutoDelete( TRUE );
     delete fontDict;
-    delete fontCache;				// delete font cache
     delete fontNameDict;
 }
 
@@ -222,11 +257,11 @@ void QFont::cacheStatistics()
 #if defined(DEBUG)
     fontCache->statistics();
     QFontCacheIt it(*fontCache);
-    QXFontData *tmp;
+    QFontInternal *fin;
     debug( "{" );
-    while ( (tmp = it.current()) ) {
-	debug( "   [%s]",tmp->name.data() );
+    while ( (fin = it.current()) ) {
 	++it;
+	debug( "   [%s]", fin->name() );
     }
     debug( "}" );
 #endif
@@ -238,40 +273,37 @@ void QFont::cacheStatistics()
   Constructs a font object that refers to the default font.
 */
 
-QFont::QFont( bool )				// create default font
+QFont::QFont( bool )
 {
     init();
-    d->req.family  = "fixed";
-    d->req.pointSize = 12*10;
+    d->req.family    = "6x13";
+    d->req.pointSize = 11*10;
     d->req.weight    = QFont::Normal;
-    d->req.rawMode = TRUE;
+    d->req.rawMode   = TRUE;
 }
 
 
 // If d->req.dirty is not TRUE the font must have been loaded
-// and we can safely assume that d->xfd is a valid pointer:
+// and we can safely assume that d->fin is a valid pointer:
 
-#define DIRTY_FONT	(d->req.dirty	|| d->xfd->dirty())
-#define DIRTY_METRICS	(f.d->req.dirty || f.d->xfd->dirty())
+#define DIRTY_FONT (d->req.dirty || d->fin->dirty())
 
 
 /*!
-  Returns a window system handle to the font.
-
-  Use of this function is discouraged at present.
+  Returns the window system handle to the font, for low-level
+  access.  <em>Using this function is not portable.</em>
 */
 
 HANDLE QFont::handle( HANDLE ) const
 {
     static Font last = 0;
-    if ( DIRTY_FONT )
-	loadFont();
-    else {
-	if ( d->xfd->f->fid != last ) {
-	    fontCache->find( d->xfd->name );
-	}
+    if ( DIRTY_FONT ) {
+	load();
+    } else {
+	if ( d->fin->fontStruct()->fid != last )
+	    fontCache->find( d->fin->name() );
     }
-    last = d->xfd->f->fid;
+    last = d->fin->fontStruct()->fid;
     return last;
 }
 
@@ -293,17 +325,14 @@ bool QFont::dirty() const
 QString QFont::defaultFamily() const
 {
     switch( d->req.styleHint ) {
-	case Helvetica:
-	    return "helvetica";
 	case Times:
 	    return "times";
 	case Courier:
 	    return "courier";
 	case Decorative:
 	    return "old english";
+	case Helvetica:
 	case System:
-	    return "helvetica";
-	case AnyStyle:
 	default:
 	    return "helvetica";
     }
@@ -322,9 +351,11 @@ QString QFont::lastResortFamily() const
 }
 
 static const char *tryFonts[] = {
-    "fixed",
     "6x13",
+    "7x13",
+    "8x13",
     "9x15",
+    "fixed",
     "-*-helvetica-medium-r-*-*-*-120-*-*-*-*-*-*",
     "-*-courier-medium-r-*-*-*-120-*-*-*-*-*-*",
     "-*-times-medium-r-*-*-*-120-*-*-*-*-*-*",
@@ -353,22 +384,23 @@ QString QFont::lastResortFont() const
     static const char *last = 0;
     if ( last )					// already found
 	return last;
-    const char *tmp = tryFonts[0];
-    while ( tmp ) {
-	if ( fontExists( tmp ) ) {
-	    last = tmp;
+    int i = 0;
+    const char *f;
+    while ( (f = tryFonts[i]) ) {
+	if ( fontExists(f) ) {
+	    last = f;
 	    return last;
 	}
-	tmp++;
+	i++;
     }
 #if defined(CHECK_NULL)
-    fatal( "QFont::lastResortFont: Cannot find any reasonable font!" );
+    fatal( "QFont::lastResortFont: Cannot find any reasonable font" );
 #endif
     return last;
 }
 
 
-static void resetFontDef( QFontDef *def )	// used by updateFontInfo()
+static void resetFontDef( QFontDef *def )	// used by initFontInfo()
 {
     def->pointSize     = 0;
     def->styleHint     = QFont::AnyStyle;
@@ -382,112 +414,123 @@ static void resetFontDef( QFontDef *def )	// used by updateFontInfo()
 }
 
 /*!
-  Updates the font information.
+  Initializes the font information in the font's QFontInternal data.
+  This function is called from load() for a new font.
 */
 
-void QFont::updateFontInfo() const
+void QFont::initFontInfo() const
 {
-    if ( !d->act.dirty )
+    QFontInternal *f = d->fin;
+    if ( !f->s.dirty )				// already initialized
 	return;
-    if ( DIRTY_FONT )
-	loadFont();
+    f->lw = computeLineWidth( f->name() );
     if ( exactMatch() ) {			// match: copy font description
-	d->act = d->req;
+	f->s = d->req;
+	f->s.dirty = FALSE;
 	return;
     }
+ 
     char *tokens[fontFields];
-    QString buffer( 256 );			// holds parsed X font name
-
-    buffer = d->xfd->name.copy();
-    if ( !parseXFontName( buffer, tokens ) ) {
-	resetFontDef( &d->act );
-	d->exactMatch  = FALSE;
-	d->act.family  = d->xfd->name;
-	d->act.rawMode = TRUE;
-	d->act.dirty   = FALSE;
-	return;					// not an XLFD name
+    QString buffer = f->name();
+    if ( !parseXFontName(buffer, tokens) ) {	// not an XLFD name
+	resetFontDef( &f->s );
+	f->s.family   = f->name();
+	f->s.rawMode  = TRUE;
+	d->exactMatch = FALSE;
+	return;
     }
-
-    d->act.family      = tokens[Family];
-    d->act.pointSize   = atoi( tokens[PointSize] );
-    d->act.styleHint   = QFont::AnyStyle;    // ### any until we match families
+    f->s.family    = tokens[Family];
+    f->s.pointSize = atoi(tokens[PointSize]);
+    f->s.styleHint = QFont::AnyStyle;	// ### any until we match families
 
     if ( strcmp( tokens[CharsetRegistry], "iso8859" ) == 0 &&
 	 strcmp( tokens[CharsetEncoding], "1"	    ) == 0 )
-	d->act.charSet = QFont::Latin1;
+	f->s.charSet = QFont::Latin1;
     else
-	d->act.charSet = QFont::AnyCharSet;
+	f->s.charSet = QFont::AnyCharSet;
 
-    char slant	       = tolower( tokens[Slant][0] );
-    d->act.italic      = ( slant == 'o' || slant == 'i' ) ? TRUE : FALSE;
+    char slant	= tolower( tokens[Slant][0] );
+    f->s.italic = (slant == 'o' || slant == 'i');
+    char fixed	= tolower( tokens[Spacing][0] );
+    f->s.fixedPitch = (fixed == 'm' || fixed == 'c');
+    f->s.weight = getWeight( tokens[Weight_] );
 
-    char fixed	       = tolower( tokens[Spacing][0] );
-    d->act.fixedPitch  = ( fixed == 'm' || fixed == 'c' ) ? TRUE : FALSE;
-
-    d->act.weight      = getWeight( tokens[Weight_] );
-
-    if ( strcmp( tokens[ResolutionY], "75") != 0 ) {	// if not 75 dpi
-	d->act.pointSize = ( 2*d->act.pointSize*atoi(tokens[ResolutionY]) + 1 )
-			  / ( 75 * 2 );		// adjust actual pointsize
+    if ( strcmp(tokens[ResolutionY], "75") != 0 ) { // if not 75 dpi
+	f->s.pointSize = ( 2*f->s.pointSize*atoi(tokens[ResolutionY]) + 1)
+			   / (75 * 2);		// adjust actual pointsize
     }
-    d->act.underline = d->req.underline;
-    d->act.strikeOut = d->req.strikeOut;
-    d->act.rawMode = FALSE;
-    d->act.dirty   = FALSE;
+    f->s.underline = d->req.underline;
+    f->s.strikeOut = d->req.strikeOut;
+    f->s.rawMode = FALSE;
+    f->s.dirty   = FALSE;
 }
 
 
 /*!
-  Loads the requested font.
+  Loads the font.
 */
 
-void QFont::loadFont( HANDLE ) const
+void QFont::load( HANDLE ) const
 {
-    QString instanceID;
-    QXFontName	*fn;
-
-    if ( !fontNameDict || !fontCache )		// not initialized
+    if ( !fontCache )				// not initialized
 	return;
 
-    if ( d->req.rawMode )
-	instanceID = "&" + d->req.family;
-    else
-	instanceID.sprintf( "x%s_%i_%i_%i_%i_%i_%i",
-			    family(),deciPointSize(),
-			    d->req.hintSetByUser ? styleHint() :
-						   QFont::AnyStyle,
-			    charSet(), weight(), italic() ? 1 : 0,
-			    fixedPitch() ? 1 : 0 );
+    QString k = key();
+    QXFontName *fn = fontNameDict->find( k );
 
-    fn = fontNameDict->find( instanceID );
     if ( !fn ) {
-	fn = new QXFontName;
+	QString name;
+	bool match;
 	if ( d->req.rawMode ) {
-	    if ( fontExists( family() ) ) {
-		fn->name       = family();
-		fn->exactMatch = TRUE;
-	    } else {
-		fn->name       = lastResortFont();
-		fn->exactMatch = FALSE;
-	    }
+	    name = substitute( family() );
+	    match = fontExists( name );
+	    if ( !match )
+		name = lastResortFont();
 	} else {
-	    fn->name = PRIV->findFont( &fn->exactMatch ); // returns a
-							  // loadable font.
+	    name = PRIV->findFont( &match );
 	}
-	fontNameDict->insert( instanceID, fn );
+	fn = new QXFontName( name, match );
+	CHECK_PTR( fn );
+	fontNameDict->insert( k, fn );
+    }
+
+    QString n = fn->name;
+    d->fin = fontCache->find( n );
+    if ( !d->fin ) {				// font not loaded
+	d->fin = fontDict->find( n );
+	if ( !d->fin ) {			// font was never loaded
+	    d->fin = new QFontInternal( n );
+	    CHECK_PTR( d->fin );
+	    fontDict->insert( n, d->fin );
+	}
+    }
+    XFontStruct *f = d->fin->f;
+    if ( !f ) {					// font not loaded
+	f = XLoadQueryFont( QPaintDevice::x__Display(), n );
+	if ( !f ) {
+	    f = XLoadQueryFont( QPaintDevice::x__Display(), lastResortFont());
+	    fn->exactMatch = FALSE;
+#if defined(CHECK_NULL)
+	    if ( !f )
+		fatal( "QFont::load: Internal error" );
+#endif
+	}
+	int size = (f->max_bounds.ascent + f->max_bounds.descent) *
+		   f->max_bounds.width *
+		   (f->max_char_or_byte2 - f->min_char_or_byte2) / 8;
+	// If we get a cache overflow, we make room for this font only
+	if ( size > fontCache->maxCost() + reserveCost )
+	    fontCache->setMaxCost( size + reserveCost );
+	if ( !fontCache->insert(n, d->fin, size) ) {
+#if defined(DEBUG)
+	    fatal( "QFont::load: Cache overflow error" );
+#endif
+	}
+	d->fin->f = f;
+	initFontInfo();
     }
-    bool newFont;
     d->exactMatch = fn->exactMatch;
-    d->xfd	  = loadXFont( fn->name, &newFont );
-    if ( !d->xfd ) {
-	d->exactMatch = FALSE;
-	d->xfd = loadXFont( lastResortFont(), &newFont );
-	if ( !d->xfd )
-	    fatal( "QFont::loadFont: Internal error" );
-    }
-    d->lineW	 = computeLineWidth( d->xfd->name );
     d->req.dirty = FALSE;
-    d->act.dirty = TRUE;	// actual font information no longer valid
 }
 
 
@@ -510,9 +553,10 @@ void QFont::loadFont( HANDLE ) const
 // of a font.
 //
 
-int QFont_Private::fontMatchScore( char	 *fontName, QString &buffer,
+int QFont_Private::fontMatchScore( char	 *fontName,	 QString &buffer,
 				   float *pointSizeDiff, int  *weightDiff,
-				   bool	 *scalable     , bool *polymorphic )
+				   bool	 *scalable     , bool *polymorphic,
+                                   int   *resx	       , int  *resy )
 {
     char *tokens[fontFields];
     bool   exactMatch = TRUE;
@@ -573,19 +617,26 @@ int QFont_Private::fontMatchScore( char	 *fontName, QString &buffer,
     }
 
     int pSize;
-
-    if ( *scalable ) {
+    if ( *scalable )
 	pSize = deciPointSize();	// scalable font
-	score |= ResolutionScore;	// know we can ask for 75x75 resolution
-    } else {
+    else
 	pSize = atoi( tokens[PointSize] );
-	if ( strcmp(tokens[ResolutionX], "75") == 0 &&
-	     strcmp(tokens[ResolutionY], "75") == 0 ) {
-	    score |= ResolutionScore;
-	} else {
-	    exactMatch = FALSE;
-	    pSize = (2*pSize*atoi(tokens[ResolutionY]) + 75) / (75*2);
+
+    if ( strcmp(tokens[ResolutionX], "0") == 0 &&
+	 strcmp(tokens[ResolutionY], "0") == 0 ) {
+	// smoothly scalable font, we can ask for any resolution
+	score |= ResolutionScore;
+    } else {
+	int localResx = atoi(tokens[ResolutionX]);
+	int localResy = atoi(tokens[ResolutionY]);
+	if ( *resx == 0 || *resy == 0 ) {
+	    *resx = localResx;
+	    *resy = localResy;
 	}
+	if ( localResx == *resx && localResy == *resy ) 
+	    score |= ResolutionScore;
+	else
+	    exactMatch = FALSE;
     }
 
     float diff;
@@ -640,7 +691,7 @@ struct MatchData {			// internal for bestMatch
     int	    weightDiff;
 };
 
-QString QFont_Private::bestMatch( const QString &pattern, int *score )
+QString QFont_Private::bestMatch( const char *pattern, int *score )
 {
     MatchData	best;
     MatchData	bestScalable;
@@ -651,18 +702,22 @@ QString QFont_Private::bestMatch( const QString &pattern, int *score )
     int		sc;
     float	pointDiff;	// difference in % from requested point size
     int		weightDiff;	// difference from requested weight
+    int		resx        = 0;
+    int         resy	    = 0;
     bool	scalable    = FALSE;
     bool	polymorphic = FALSE;
+    int		i;
 
     xFontNames = getXFontNames( pattern, &count );
-    for( int i = 0 ; i < count ; i++ ) {
+
+    for( i = 0; i < count; i++ ) {
 	sc = fontMatchScore( xFontNames[i], matchBuffer,
 			     &pointDiff, &weightDiff,
-			     &scalable, &polymorphic );
+			     &scalable, &polymorphic, &resx, &resy );
 	if ( sc > best.score ||
-	    sc == best.score && pointDiff < best.pointDiff ||
-	    sc == best.score && pointDiff == best.pointDiff &&
-				weightDiff < best.weightDiff ) {
+	     sc == best.score && pointDiff < best.pointDiff ||
+	     sc == best.score && pointDiff == best.pointDiff &&
+				 weightDiff < best.weightDiff ) {
 	    if ( scalable ) {
 		if ( sc > bestScalable.score ||
 		     sc == bestScalable.score &&
@@ -688,10 +743,14 @@ QString QFont_Private::bestMatch( const QString &pattern, int *score )
 		 best.pointDiff != 0 ||
 		 best.weightDiff < bestScalable.weightDiff ) ) ) {
 	if ( smoothlyScalable( bestScalable.name ) ) {
+	    if ( resx == 0 || resy == 0 ) {
+		resx = 75;
+		resy = 75;
+	    }
 	    best.score = bestScalable.score;
 	    strcpy( matchBuffer.data(), bestScalable.name );
 	    if ( parseXFontName( matchBuffer, tokens ) ) {
-		bestName.sprintf( "-%s-%s-%s-%s-%s-%s-*-%i-75-75-%s-*-%s-%s",
+		bestName.sprintf( "-%s-%s-%s-%s-%s-%s-*-%i-%i-%i-%s-*-%s-%s",
 				  tokens[Foundry],
 				  tokens[Family],
 				  tokens[Weight_],
@@ -699,6 +758,7 @@ QString QFont_Private::bestMatch( const QString &pattern, int *score )
 				  tokens[Width],
 				  tokens[AddStyle],
 				  deciPointSize(),
+				  resx, resy,
 				  tokens[Spacing],
 				  tokens[CharsetRegistry],
 				  tokens[CharsetEncoding] );
@@ -707,56 +767,51 @@ QString QFont_Private::bestMatch( const QString &pattern, int *score )
 	}
     }
     *score = best.score;
-    QString tmp( best.name );
+    bestName = best.name;
     XFreeFontNames( xFontNames );
-    return tmp;
+
+    return bestName;
 }
 
 
-QString QFont_Private::bestFamilyMember( const QString &family, int *score )
+QString QFont_Private::bestFamilyMember( const char *family, int *score )
 {
-    QString pattern;
-    QString match;
-    pattern.sprintf( "-*-%s-*-*-*-*-*-*-*-*-*-*-*-*", family.data() );
+    char pattern[256];
+    sprintf( pattern, "-*-%s-*-*-*-*-*-*-*-*-*-*-*-*", family );
     return bestMatch( pattern, score );
 }
 
 
 QString QFont_Private::findFont( bool *exact )
 {
-    QString familyName;
-    QString bestName;
-    int	    score;
-    const char *fam = family();			// fam = font family name
-
-    if ( fam == 0 || fam[0] == '\0' ) {		// null or empty string
+    QString familyName = substitute( family() );
+    *exact = TRUE;				// assume exact match
+    if ( familyName.isEmpty() ) {
 	familyName = defaultFamily();
-	*exact	   = FALSE;
-    } else {
-	familyName = bestFitFamily( fam );
-	*exact	   = TRUE;
+	*exact = FALSE;
     }
-    bestName = bestFamilyMember( familyName, &score );
-    if ( *exact && score != exactScore )
+
+    int score;
+    QString bestName = bestFamilyMember( familyName, &score );
+    if ( score != exactScore )
 	*exact = FALSE;
 
     if ( score == 0 ) {
-	QString df = defaultFamily();
-	if( familyName != df ) {
-	    familyName = df;			// try default family for style
-	    bestName   = bestFamilyMember( familyName, &score );
+	QString f = defaultFamily();
+	if( familyName != f ) {
+	    familyName = f;			// try default family for style
+	    bestName = bestFamilyMember( familyName, &score );
+	}
+	if ( score == 0 ) {
+	    f = lastResortFamily();
+	    if ( familyName != f ) {
+		familyName = f;			// try system default family
+		bestName = bestFamilyMember( familyName, &score );
+	    }
 	}
     }
-    if ( score == 0 ) {
-	QString lrf = lastResortFamily();
-	if ( familyName != lrf ) {
-	    familyName = lrf;			// try system default family
-	    bestName   = bestFamilyMember( familyName, &score );
-	}
-    }
-    if ( bestName.isNull() ) {			// no matching fonts found
+    if ( bestName.isNull() )			// no matching fonts found
 	bestName = lastResortFont();
-    }
     return bestName;
 }
 
@@ -765,6 +820,53 @@ QString QFont_Private::findFont( bool *exact )
   QFontMetrics member functions
  *****************************************************************************/
 
+const QFontDef *QFontMetrics::spec() const
+{
+    const QFontDef *s;
+    if ( type() == FontInternal ) {
+	s = u.f->spec();
+    } else if ( type() == Widget && u.w ) {
+	QFont *f = (QFont *)&u.w->font();
+	f->handle();
+	s = f->d->fin->spec();
+    } else if ( type() == Painter && u.p ) {
+	QFont *f = (QFont *)&u.p->font();
+	f->handle();
+	s = f->d->fin->spec();
+    } else {
+	s = 0;
+    }
+#if defined(CHECK_NULL)
+    if ( !s )
+	warning( "QFontMetrics: Invalid font metrics" );
+#endif
+    return s;
+}
+
+void *QFontMetrics::fontStruct() const
+{
+    if ( type() == FontInternal ) {
+	return u.f->fontStruct();
+    } else if ( type() == Widget && u.w ) {
+	QFont *f = (QFont *)&u.w->font();
+	f->handle();
+	return f->d->fin->fontStruct();
+    } else if ( type() == Painter && u.p ) {
+	QFont *f = (QFont *)&u.p->font();
+	f->handle();
+	return f->d->fin->fontStruct();
+    } else {
+#if defined(CHECK_NULL)
+	warning( "QFontMetrics: Invalid font metrics" );
+#endif
+	return 0;
+    }
+}
+
+#undef  FS
+#define FS (type() == FontInternal ? u.f->fontStruct() : (XFontStruct*)fontStruct())
+
+
 /*!
   Returns the maximum ascent of the font.
 
@@ -776,15 +878,7 @@ QString QFont_Private::findFont( bool *exact )
 
 int QFontMetrics::ascent() const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return 0;
-    }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
-    return f.d->xfd->f->max_bounds.ascent;
+    return FS->max_bounds.ascent;
 }
 
 
@@ -800,15 +894,7 @@ int QFontMetrics::ascent() const
 
 int QFontMetrics::descent() const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return 0;
-    }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
-    return f.d->xfd->f->max_bounds.descent - 1;
+    return FS->max_bounds.descent - 1;
 }
 
 
@@ -822,15 +908,8 @@ int QFontMetrics::descent() const
 
 int QFontMetrics::height() const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return 0;
-    }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
-    return f.d->xfd->f->max_bounds.ascent + f.d->xfd->f->max_bounds.descent;
+    XFontStruct *f = FS;
+    return f->max_bounds.ascent + f->max_bounds.descent;
 }
 
 
@@ -844,16 +923,9 @@ int QFontMetrics::height() const
 
 int QFontMetrics::leading() const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return 0;
-    }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
-    int l = f.d->xfd->f->ascent		   + f.d->xfd->f->descent -
-	    f.d->xfd->f->max_bounds.ascent - f.d->xfd->f->max_bounds.descent;
+    XFontStruct *f = FS;
+    int l = f->ascent		 + f->descent -
+	    f->max_bounds.ascent - f->max_bounds.descent;
     return l > 0 ? l : 0;
 }
 
@@ -889,17 +961,9 @@ int QFontMetrics::lineSpacing() const
 
 int QFontMetrics::width( const char *str, int len ) const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return 0;
-    }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
     if ( len < 0 )
 	len = strlen( str );
-    return XTextWidth( f.d->xfd->f, str, len );
+    return XTextWidth( FS, str, len );
 }
 
 
@@ -916,28 +980,37 @@ int QFontMetrics::width( const char *str, int len ) const
 
 QRect QFontMetrics::boundingRect( const char *str, int len ) const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return QRect(0,0,0,0);
-    }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
     if ( len < 0 )
 	len = strlen( str );
-    XFontStruct *fs = f.d->xfd->f;
+    XFontStruct    *f = FS;
     int direction;
     int ascent;
     int descent;
     XCharStruct overall;
 
-    XTextExtents( fs, str, len, &direction, &ascent, &descent, &overall );
+    bool underline;
+    bool strikeOut;
+    if ( type() == FontInternal ) {
+	underline = underlineFlag();
+	strikeOut = strikeOutFlag();
+    } else if ( type() == Widget && u.w ) {	
+	const QFont &f = u.w->font();
+	underline = f.underline();
+	strikeOut = f.strikeOut();
+    } else if ( type() == Painter && u.p ) {
+	const QFont &f = u.p->font();
+	underline = f.underline();
+	strikeOut = f.strikeOut();
+    } else {
+	underline = strikeOut = FALSE;
+    }
+
+    XTextExtents( f, str, len, &direction, &ascent, &descent, &overall );
     int startX = overall.lbearing;
     int width  = overall.rbearing - startX;
     ascent     = overall.ascent;
     descent    = overall.descent;
-    if ( !f.d->req.underline && !f.d->req.strikeOut ) {
+    if ( !underline && !strikeOut ) {
 	width  = overall.rbearing - startX;
     } else {
 	if ( startX > 0 )
@@ -946,7 +1019,7 @@ QRect QFontMetrics::boundingRect( const char *str, int len ) const
 	   width =  overall.width - startX;
 	else
 	   width =  overall.rbearing - startX;
-	if ( f.d->req.underline && len != 0 ) {
+	if ( underline && len != 0 ) {
 	    int ulTop = underlinePos();
 	    int ulBot = ulTop + lineWidth(); // X descent is 1
 	    if ( descent < ulBot )	// more than logical descent, so don't
@@ -954,7 +1027,7 @@ QRect QFontMetrics::boundingRect( const char *str, int len ) const
 	    if ( ascent < -ulTop )
 		ascent = -ulTop;
 	}
-	if ( f.d->req.strikeOut && len != 0 ) {
+	if ( strikeOut && len != 0 ) {
 	    int soTop = strikeOutPos();
 	    int soBot = soTop - lineWidth(); // same as above
 	    if ( descent < -soBot )
@@ -973,15 +1046,7 @@ QRect QFontMetrics::boundingRect( const char *str, int len ) const
 
 int QFontMetrics::maxWidth() const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return 0;
-    }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
-    return f.d->xfd->f->max_bounds.width;
+    return FS->max_bounds.width;
 }
 
 
@@ -993,7 +1058,7 @@ int QFontMetrics::maxWidth() const
 
 int QFontMetrics::underlinePos() const
 {
-    int pos = ( lineWidth()*2 + 3 )/6;
+    int pos = (lineWidth()*2 + 3)/6;
     return pos ? pos : 1;
 }
 
@@ -1006,15 +1071,7 @@ int QFontMetrics::underlinePos() const
 
 int QFontMetrics::strikeOutPos() const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return 0;
-    }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
-    int pos = f.d->xfd->f->max_bounds.ascent/3;
+    int pos = FS->max_bounds.ascent/3;
     return pos ? pos : 1;
 }
 
@@ -1027,15 +1084,41 @@ int QFontMetrics::strikeOutPos() const
 
 int QFontMetrics::lineWidth() const
 {
-    if ( w == 0 && p == 0 ) {
-#if defined(CHECK_STATE)
-	warning( "QFontMetrics: Invalid font metrics" );
-#endif
-	return 0;
+    if ( type() == FontInternal ) {
+	return u.f->lineWidth();
+    } else {
+	QFont f = font();
+	f.handle();
+	return f.d->fin->lineWidth();
     }
-    QFont f = w ? w->font() : p->font();
-    f.handle();
-    return f.d->lineW;
+}
+
+
+/*****************************************************************************
+  QFontInfo member functions
+ *****************************************************************************/
+
+const QFontDef *QFontInfo::spec() const
+{
+    const QFontDef *s;
+    if ( type() == FontInternal ) {
+	s = u.f->spec();
+    } else if ( type() == Widget && u.w ) {
+	QFont *f = (QFont *)&u.w->font();
+	f->handle();
+	s = f->d->fin->spec();
+    } else if ( type() == Painter && u.p ) {
+	QFont *f = (QFont *)&u.p->font();
+	f->handle();
+	s = f->d->fin->spec();
+    } else {
+	s = 0;
+    }
+#if defined(CHECK_NULL)
+    if ( !s )
+	warning( "QFontInfo: Invalid font info" );
+#endif
+    return s;
 }
 
 
@@ -1043,52 +1126,6 @@ int QFontMetrics::lineWidth() const
   Internal X font functions
  *****************************************************************************/
 
-//
-// Loads an X font
-//
-
-static QXFontData *loadXFont( const QString &fontName, bool *newFont )
-{
-    if ( !fontCache )
-	return 0;
-
-    if ( newFont )
-	*newFont = FALSE;
-    QXFontData *xfd = fontCache->find( fontName );
-    if ( xfd )
-	return xfd;
-						// font is not cached
-    QXFontStruct  *tmp;
-    tmp = XLoadQueryFont( qt_xdisplay(), fontName );
-    if ( !tmp )
-	return 0;				// could not load font
-
-    xfd = fontDict->find( fontName );
-    if ( !xfd ) {				// first time font is loaded?
-	xfd = new QXFontData;
-	CHECK_PTR( xfd );
-	fontDict->insert( fontName, xfd );
-    } else {
-	if ( newFont )
-	    *newFont = TRUE;
-    }
-    xfd->f    = tmp;
-    xfd->name = fontName;			// used by QFontInfo
-    XFontStruct *f = xfd->f;
-    int sz = (f->max_bounds.ascent + f->max_bounds.descent)
-	      *f->max_bounds.width
-	      *(f->max_char_or_byte2 - f->min_char_or_byte2) / 8;
-    if ( sz > fontCache->maxCost() + reserveCost )	// make room for this
-	fontCache->setMaxCost( sz + reserveCost );	//   font only
-    if ( !fontCache->insert( fontName, xfd ,sz ) ) {
-#if defined(DEBUG)
-	fatal( "loadXFont(): internal error; cache overflow" );
-#endif
-    }
-    return xfd;
-}
-
-
 //
 // Splits an X font name into fields separated by '-'
 //
@@ -1100,12 +1137,12 @@ static bool parseXFontName( QString &fontName, char **tokens )
 	return FALSE;
     }
     int	  i;
-    char *tmp = fontName.data() + 1;
-    for ( i=0; i<fontFields && tmp && tmp[0]; i++ ) {
-	tokens[i] = tmp;
-	tmp = strchr( tmp, '-' );
-	if( tmp )
-	    *tmp++ = '\0';
+    char *f = fontName.data() + 1;
+    for ( i=0; i<fontFields && f && f[0]; i++ ) {
+	tokens[i] = f;
+	f = strchr( f, '-' );
+	if( f )
+	    *f++ = '\0';
     }
     if ( i < fontFields ) {
 	for( int j=i ; j<fontFields; j++ )
@@ -1116,18 +1153,6 @@ static bool parseXFontName( QString &fontName, char **tokens )
 }
 
 
-//
-// Maps a font family name to a valid X family name
-//
-
-static QString bestFitFamily( const QString &fam )
-{
-    QString family = QFont::substitute( fam );
-    family.lower();
-    return family;
-}
-
-
 //
 // Get an array of X font names that matches a pattern
 //
@@ -1135,12 +1160,13 @@ static QString bestFitFamily( const QString &fam )
 static char **getXFontNames( const char *pattern, int *count )
 {
     static int maxFonts = 256;
-    char **tmp;
+    char **list;
     while( 1 ) {
-	tmp = XListFonts( qt_xdisplay(), (char*) pattern, maxFonts, count );
+	list = XListFonts( QPaintDevice::x__Display(), (char*)pattern,
+			   maxFonts, count );
 	if ( *count != maxFonts || maxFonts >= 32768 )
-	    return tmp;
-	XFreeFontNames( tmp );
+	    return list;
+	XFreeFontNames( list );
 	maxFonts *= 2;
     }
 }
@@ -1164,10 +1190,9 @@ static bool fontExists( const char *fontName )
 {
     char **fontNames;
     int	   count;
-    fontNames	 = getXFontNames( fontName, &count );
-    bool success = ( count != 0 );
+    fontNames = getXFontNames( fontName, &count );
     XFreeFontNames( fontNames );
-    return success;
+    return count != 0;
 }
 
 
@@ -1199,40 +1224,52 @@ static int computeLineWidth( const char *fontName )
 // Converts a weight string to a value
 //
 
+Q_DECLARE(QDictM,int);
+QDictM(int) *weightDict = 0;
+
+static void cleanupWeightDict()
+{
+    delete weightDict;
+    weightDict = 0;
+}
+
 static int getWeight( const char *weightString, bool adjustScore )
 {
-    QString wStr = weightString;
-    wStr.lower();
-    if ( wStr == "medium" )
-	return (int) QFont::Normal;
-    if ( wStr == "bold" )
-	return (int) QFont::Bold;
-    if ( wStr == "demibold" )
-	return (int) QFont::DemiBold;
-    if ( wStr == "black" )
-	return (int) QFont::Black;
-    if ( wStr == "light" )
-	return (int) QFont::Light;
-    if ( wStr.contains( "bold" ) ) {
-	if( adjustScore )
+    if ( !weightDict ) {
+	qAddPostRoutine( cleanupWeightDict );
+	weightDict = new QDictM(int)( 17, FALSE ); // case insensitive
+	CHECK_PTR( weightDict );
+	weightDict->insert( "medium",   (int*)((int)QFont::Normal+1) );
+	weightDict->insert( "bold",     (int*)((int)QFont::Bold+1) );
+	weightDict->insert( "demibold", (int*)((int)QFont::DemiBold+1) );
+	weightDict->insert( "black",    (int*)((int)QFont::Black+1) );
+	weightDict->insert( "light",    (int*)((int)QFont::Light+1) );
+    }
+    int *p = weightDict->find(weightString);
+    if ( p )
+	return (int)p - 1;
+    QString s = weightString;
+    s = s.lower();
+    if ( s.contains("bold") ) {
+	if ( adjustScore )
 	    return (int) QFont::Bold - 1;  // - 1, not sure that this IS bold
-       else
+	else
 	    return (int) QFont::Bold;
     }
-    if ( wStr.contains( "light" ) ) {
-	if( adjustScore )
+    if ( s.contains("light") ) {
+	if ( adjustScore )
 	    return (int) QFont::Light - 1; // - 1, not sure that this IS light
        else
 	    return (int) QFont::Light;
     }
-    if ( wStr.contains( "black" ) ) {
-	if( adjustScore )
+    if ( s.contains("black") ) {
+	if ( adjustScore )
 	    return (int) QFont::Black - 1; // - 1, not sure this IS black
-       else
+	else
 	    return (int) QFont::Black;
     }
     if ( adjustScore )
-	return (int) QFont::Normal - 2; // - 2, we hope it's close to normal
+	return (int) QFont::Normal - 2;	   // - 2, we hope it's close to normal
     else
 	return (int) QFont::Normal;
 }
