@@ -1,17 +1,19 @@
 /**********************************************************************
-** $Id: qpsprn.cpp,v 2.0 1996/09/24 10:15:59 agulbra Exp $
+** $Id: qpsprn.cpp,v 2.2 1997/03/10 13:26:30 eiriken Exp $
 **
 ** Implementation of QPSPrinter class
 **
 ** Created : 941003
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -24,7 +26,7 @@
 #include "qfile.h"
 #include "qbuffer.h"
 
-RCSTAG("$Id: qpsprn.cpp,v 2.0 1996/09/24 10:15:59 agulbra Exp $");
+RCSTAG("$Id: qpsprn.cpp,v 2.2 1997/03/10 13:26:30 eiriken Exp $");
 
 
 #if !defined(QT_HEADER_PS)
@@ -126,18 +128,17 @@ static void ps_setFont( QTextStream *s, const QFont *f, QString *fonts )
     bool times	= FALSE;
     bool symbol = FALSE;
     family = family.lower();
-    if ( family == "courier" )
+    if ( family == "courier" ) {
 	ps = "/Courier";
-    else if ( family == "times" ) {
+    } else if ( family == "times" ) {
 	ps = "/Times";
 	times = TRUE;
-    }
-    else if ( family == "symbol" ) {
+    } else if ( family == "symbol" ) {
 	ps = "/Symbol";
 	symbol = TRUE;
-    }
-    else
+    } else {
 	ps = "/Helvetica";
+    }
     QString extra;
     if ( weight >= QFont::Bold && !symbol )
 	extra = "Bold";
@@ -150,15 +151,14 @@ static void ps_setFont( QTextStream *s, const QFont *f, QString *fonts )
     if ( !extra.isEmpty() ) {
 	ps += '-';
 	ps += extra;
-    }
-    else {
+    } else {
 	if ( times )
 	    ps += "-Roman";
     }
     QString fontMatrix;
     fontMatrix.sprintf( "[ %d 0 0 -%d 0 0 ]", f->pointSize(), f->pointSize() );
     *s << ps << " findfont " << fontMatrix << " makefont setfont\n";
-    ps.remove( 0, 1 );	// removes the '/'
+    ps.remove( 0, 1 );				// removes the '/'
     ps += ' ';
     if ( !fonts->contains(ps) )
 	*fonts += ps;
@@ -201,7 +201,7 @@ static void ps_dumpPixmapData( QTextStream &stream, QImage img,
     stream.setf( QTextStream::hex, QTextStream::basefield );
 
     if ( img.depth() == 1 ) {
-	img.convertDepth( 8 );
+	img = img.convertDepth( 8 );
 	if ( img.color(0) == 0 ) {			// black
 	    img.setColor( 0, fgCol.rgb() );
 	    img.setColor( 1, bgCol.rgb() );
@@ -498,17 +498,14 @@ bool QPSPrinter::cmd( int c , QPainter *paint, QPDevCmdParam *p )
 	    ps_setFont( &stream, p[0].font, &fontsUsed );
 	    break;
 	case PDC_SETPEN:
-	    if ( p[0].pen->width() == 0 )
-		stream << (int)p[0].pen->style()   << " 0.3 "
-		       << COLOR(p[0].pen->color()) << "PE\n";
-	    else
-		stream << (int)p[0].pen->style()   << ' '
-		       << p[0].pen->width()
-		       << COLOR(p[0].pen->color()) << "PE\n";
+	    stream << (int)p[0].pen->style() << ' ' << p[0].pen->width()
+		   << ' ' << COLOR(p[0].pen->color()) << "PE\n";
 	    break;
 	case PDC_SETBRUSH:
 	    if ( p[0].brush->style() == CustomPattern ) {
+#if defined(DEBUG)
 		warning( "QPrinter: Pixmap brush not supported" );
+#endif
 		return FALSE;
 	    }
 	    stream << (int)p[0].brush->style()	 << ' '
