@@ -1,25 +1,25 @@
 /****************************************************************************
-** $Id: qdatetm.cpp,v 2.0 1996/09/24 10:16:08 agulbra Exp $
+** $Id: qdatetm.cpp,v 2.3 1997/02/03 14:20:15 hanord Exp $
 **
 ** Implementation of date and time classes
 **
 ** Created : 940124
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
+#define gettimeofday	__hide_gettimeofday
 #include "qdatetm.h"
 #include "qdstream.h"
-#if defined(UNIX)
-#define gettimeofday	__hide_gettimeofday
-#endif
 #include <stdio.h>
 #include <time.h>
 #if defined(_OS_WIN32_)
@@ -41,7 +41,7 @@
 extern "C" int gettimeofday( struct timeval *, struct timezone * );
 #endif
 
-RCSTAG("$Id: qdatetm.cpp,v 2.0 1996/09/24 10:16:08 agulbra Exp $");
+RCSTAG("$Id: qdatetm.cpp,v 2.3 1997/02/03 14:20:15 hanord Exp $");
 
 
 static const uint FIRST_DAY	= 2361222;	// Julian day for 17520914
@@ -862,6 +862,15 @@ void QDateTime::setTime_t( uint secsSince1Jan1970UTC )
 {
     time_t tmp = (time_t) secsSince1Jan1970UTC;
     tm *tM = localtime( &tmp );
+    if ( !tM ) {
+	tM = localtime( 0 );
+	if ( !tM ) {
+#if defined(CHECK_NULL)
+	    warning( "QDateTime::setTime_t: Cannot get localtime" );
+#endif
+	    return;
+	}
+    }
     d.jd = QDate::greg2jul( tM->tm_year + 1900, tM->tm_mon + 1, tM->tm_mday );
     t.ds = MSECS_PER_HOUR*tM->tm_hour + MSECS_PER_MIN*tM->tm_min +
 	    1000*tM->tm_sec;
@@ -1025,12 +1034,12 @@ QDateTime QDateTime::currentDateTime()
   \relates QDate
   Writes the date to the stream.
 
-  Serialization format: [UINT32], Julian day.
+  Serialization format: [Q_UINT32], Julian day.
 */
 
 QDataStream &operator<<( QDataStream &s, const QDate &d )
 {
-    return s << (UINT32)(d.jd);
+    return s << (Q_UINT32)(d.jd);
 }
 
 /*!
@@ -1040,7 +1049,7 @@ QDataStream &operator<<( QDataStream &s, const QDate &d )
 
 QDataStream &operator>>( QDataStream &s, QDate &d )
 {
-    UINT32 jd;
+    Q_UINT32 jd;
     s >> jd;
     d.jd = jd;
     return s;
@@ -1050,12 +1059,12 @@ QDataStream &operator>>( QDataStream &s, QDate &d )
   \relates QTime
   Writes a time to the stream.
 
-  Serialization format: [UINT32], milliseconds since midnight.
+  Serialization format: [Q_UINT32], milliseconds since midnight.
 */
 
 QDataStream &operator<<( QDataStream &s, const QTime &t )
 {
-    return s << (UINT32)(t.ds);
+    return s << (Q_UINT32)(t.ds);
 }
 
 /*!
@@ -1065,7 +1074,7 @@ QDataStream &operator<<( QDataStream &s, const QTime &t )
 
 QDataStream &operator>>( QDataStream &s, QTime &t )
 {
-    UINT32 ds;
+    Q_UINT32 ds;
     s >> ds;
     t.ds = ds;
     return s;
