@@ -1,17 +1,19 @@
 /****************************************************************************
-** $Id: qdir.cpp,v 2.0 1996/09/24 10:16:09 agulbra Exp $
+** $Id: qdir.cpp,v 2.3 1997/03/30 15:22:50 hanord Exp $
 **
 ** Implementation of QDir class
 **
 ** Created : 950427
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -29,8 +31,12 @@
 #include <windows.h>
 #endif
 #endif
+#if defined(_OS_OS2EMX_)
+#include <ctype.h>
+#define chdir _chdir2
+#endif
 
-RCSTAG("$Id: qdir.cpp,v 2.0 1996/09/24 10:16:09 agulbra Exp $");
+RCSTAG("$Id: qdir.cpp,v 2.3 1997/03/30 15:22:50 hanord Exp $");
 
 
 #if defined(_OS_FATFS_)
@@ -1101,6 +1107,10 @@ QString QDir::rootDirPath()
 {
 #if defined(_OS_FATFS_)
     QString d( "c:/" );
+#elif defined(_OS_OS2EMX_)
+    char dir[4];
+    _abspath( dir, "/", _MAX_PATH );
+    QString d( dir );
 #elif defined(UNIX)
     QString d( "/" );
 #else
@@ -1140,7 +1150,7 @@ QString QDir::cleanDirPath( const char *filePath )
     convertSeparators( name.data() );
 
     bool addedSeparator;
-    if ( name.at( 0 ) != '/' ) {
+    if ( isRelativePath(filePath) ) {
 	addedSeparator = TRUE;
 	name.insert( 0, '/' );
     } else {
@@ -1192,7 +1202,7 @@ bool QDir::isRelativePath( const char *path )
     int len = strlen( path );
     if ( len == 0 )
 	return TRUE;
-#if defined(_OS_FATFS_)
+#if defined(_OS_FATFS_) || defined(_OS_OS2EMX_)
     int i = 0;
     if ( isalpha(path[0]) && path[1] == ':' )		// drive, e.g. a:
 	i = 2;
@@ -1451,7 +1461,11 @@ bool QDir::readDirEntries( const QString &nameFilter,
 
 #elif defined(UNIX)
 
+#if defined(_OS_OS2EMX_)
+    QRegExp   wc( nameFilter, FALSE, TRUE );	// wild card, case insensitive
+#else
     QRegExp   wc( nameFilter, TRUE, TRUE );	// wild card, case sensitive
+#endif
     QFileInfo fi;
     DIR	     *dir;
     dirent   *file;
