@@ -1,30 +1,42 @@
 /****************************************************************************
-** $Id: qfileinf.cpp,v 2.0 1996/09/24 10:16:10 agulbra Exp $
+** $Id: qfileinf.cpp,v 2.4 1997/03/26 18:58:57 hanord Exp $
 **
 ** Implementation of QFileInfo class
 **
 ** Created : 950628
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
+#include "qglobal.h"
+#if defined(_OS_SUN_)
+#define readlink _qt_hide_readlink
+#endif
+
 #include "qfileinf.h"
 #include "qfiledef.h"
 #include "qdatetm.h"
 #include "qdir.h"
 #if defined(UNIX)
-# include <pwd.h>
-# include <grp.h>
+#include <pwd.h>
+#include <grp.h>
 #endif
 
-RCSTAG("$Id: qfileinf.cpp,v 2.0 1996/09/24 10:16:10 agulbra Exp $");
+#if defined(_OS_SUN_)
+#undef readlink
+extern "C" int readlink( const char *, void *, uint );
+#endif
+
+RCSTAG("$Id: qfileinf.cpp,v 2.4 1997/03/26 18:58:57 hanord Exp $");
 
 
 #if defined(_OS_FATFS_)
@@ -606,11 +618,11 @@ bool QFileInfo::isSymLink() const
 QString QFileInfo::readLink() const
 {
     QString s;
-#if defined(UNIX)
+#if defined(UNIX) && !defined(_OS_OS2EMX_)
     if ( !isSymLink() )
 	return s;
     s.resize( PATH_MAX+1 );
-    int len = readlink( fn, s.data(), s.size() );
+    int len = readlink( fn.data(), s.data(), s.size() );
     if ( len < 0 )
 	len = 0;				// error, return empty string
     s.truncate( len );
@@ -706,6 +718,8 @@ uint QFileInfo::groupId() const
 
 
 /*!
+  \fn bool QFileInfo::permission( int permissionSpec ) const
+
   Tests for file permissions.  The \e permissionSpec argument can be several
   flags of type PermissionSpec or'ed together to check for permission
   combinations.
@@ -725,9 +739,9 @@ uint QFileInfo::groupId() const
   \sa isReadable(), isWritable(), isExecutable()
 */
 
+#if defined(UNIX)
 bool QFileInfo::permission( int permissionSpec ) const
 {
-#if defined(UNIX)
     if ( !fic || !cache )
 	doStat();
     if ( fic ) {
@@ -761,11 +775,15 @@ bool QFileInfo::permission( int permissionSpec ) const
     } else {
 	return FALSE;
     }
-#else
-    return TRUE;
-#endif
 }
 
+#else	/* not UNIX */
+
+bool QFileInfo::permission( int ) const
+{
+    return TRUE;
+}
+#endif
 
 /*!
   Returns the file size in bytes, or 0 if the file does not exist if the size
