@@ -1,17 +1,19 @@
 /****************************************************************************
-** $Id: qregexp.cpp,v 2.0 1996/09/24 10:16:13 agulbra Exp $
+** $Id: qregexp.cpp,v 2.1 1997/03/11 12:10:31 hanord Exp $
 **
 ** Implementation of QRegExp class
 **
 ** Created : 950126
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -19,7 +21,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 
-RCSTAG("$Id: qregexp.cpp,v 2.0 1996/09/24 10:16:13 agulbra Exp $");
+RCSTAG("$Id: qregexp.cpp,v 2.1 1997/03/11 12:10:31 hanord Exp $");
 
 
 /*!
@@ -279,9 +281,9 @@ int QRegExp::match( const char *str, int index, int *len ) const
     ushort *d  = rxdata;
     char   *ep = 0;
 
-    if ( *d == BOL )				// match from beginning of line
+    if ( *d == BOL ) {				// match from beginning of line
 	ep = matchstr( d, p, p );
-    else {
+    } else {
 	if ( *d & CHR ) {
 	    if ( cs ) {				// case sensitive
 		while ( *p && *p != (char)*d )
