@@ -1,17 +1,19 @@
 /****************************************************************************
-** $Id: qtstream.cpp,v 2.0 1996/09/24 10:16:14 agulbra Exp $
+** $Id: qtstream.cpp,v 2.4 1997/01/13 12:59:55 hanord Exp $
 **
 ** Implementation of QTextStream class
 **
 ** Created : 940922
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -22,7 +24,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 
-RCSTAG("$Id: qtstream.cpp,v 2.0 1996/09/24 10:16:14 agulbra Exp $");
+RCSTAG("$Id: qtstream.cpp,v 2.4 1997/01/13 12:59:55 hanord Exp $");
 
 
 /*!
@@ -366,14 +368,22 @@ long QTextStream::input_int()
 	    break;
 	case dec:
 	    c = eat_ws( dev );
-	    if ( c == EOF )
+	    if ( c == EOF ) {
 		val = 0;
-	    else {
+	    } else {
 		if ( !(c == '-' || c == '+') )
 		    dev->ungetch( c );
-		val = (long)input_dec( this );
-		if ( c == '-' )
-		    val = -val;
+		if ( c == '-' ) {
+		    ulong v = input_dec( this );
+		    if ( v ) {		// ensure that LONG_MIN can be read
+			v--;
+			val = -((long)v) - 1;
+		    } else {
+			val = 0;
+		    }
+		} else {
+		    val = (long)input_dec( this );
+		}
 	    }
 	    break;
 	case hex:
@@ -492,7 +502,7 @@ static double input_double( QTextStream *s )
 	c = d->getch();
     }
 
-#if !defined(_CC_EDG_) // has working dead code detection
+#if !defined(NO_DEADCODE)
     return 0.0;
 #endif
 }
@@ -601,7 +611,7 @@ QTextStream &QTextStream::operator>>( double &f )
 
 
 /*!
-  Reads a string from the stream and returns a reference to the stream.
+  Reads a word from the stream and returns a reference to the stream.
 */
 
 QTextStream &QTextStream::operator>>( char *s )
@@ -625,7 +635,7 @@ QTextStream &QTextStream::operator>>( char *s )
 }
 
 /*!
-  Reads a string from the stream and returns a reference to the stream.
+  Reads a word from the stream and returns a reference to the stream.
 */
 
 QTextStream &QTextStream::operator>>( QString &s )
@@ -650,6 +660,45 @@ QTextStream &QTextStream::operator>>( QString &s )
 }
 
 
+/*!
+  Reads a line from the stream and returns a reference to the stream.
+
+  The returned string does not contain any trailing newline or carriage
+  return. Note that this is different from QIODevice::readLine(), which
+  does not strip the newline at the end of the line.
+
+  \sa QIODevice::readLine()
+*/
+
+QString QTextStream::readLine()
+{
+#if defined(CHECK_STATE)
+    if ( !dev ) {
+	warning( "QTextStream: No device" );
+	QString nullString;
+	return nullString;
+    }
+#endif
+    QString s( 64 );
+    int i = 0;
+    int c = dev->getch();
+    while ( c != EOF ) {
+	if ( c == '\n' ) {
+	    break;
+	}
+	s[i++] = c;
+	if ( i == (int)s.size()-1 )
+	    s.resize( s.size()*2 );
+	c = dev->getch();
+    }
+    if ( i > 0 && s[i-1] == '\r' )
+	i--;				// if there are two \r, let one stay
+    s.truncate(i);
+    return s;
+}
+
+
+
 /*****************************************************************************
   QTextStream write functions
  *****************************************************************************/
@@ -869,14 +918,13 @@ QTextStream &QTextStream::operator<<( double f )
     if ( prec >= 10 ) {
 	*fs++ = prec / 10 + '0';
 	*fs++ = prec % 10 + '0';
-    }
-    else
+    } else {
 	*fs++ = prec + '0';
-    ASSERT( prec > 0 && prec <10 );	// TESTING
+    }
     *fs++ = 'l';
     *fs++ = f_char;
     *fs = '\0';
-    sprintf( buf, format, f );			// the easy way out...
+    sprintf( buf, format, f );			// convert to text
     if ( fwidth )				// padding
 	*this << (const char *)buf;
     else					// just write it
@@ -890,7 +938,7 @@ QTextStream &QTextStream::operator<<( double f )
 */
 
 QTextStream &QTextStream::operator<<( const char *s )
-{						// write 0-term char array
+{
     CHECK_STREAM_PRECOND
     char padbuf[48];
     uint len = strlen( s );			// don't write null terminator
@@ -902,9 +950,9 @@ QTextStream &QTextStream::operator<<( const char *s )
 	    if ( padlen > 46 ) {		// create extra big fill buffer
 		ppad = new char[padlen];
 		CHECK_PTR( ppad );
-	    }
-	    else
+	    } else {
 		ppad = padbuf;
+	    }
 	    memset( ppad, fillchar, padlen );	// fill with fillchar
 	    if ( !(flags() & left) ) {
 		dev->writeBlock( ppad, padlen );
