@@ -1,17 +1,19 @@
 /****************************************************************************
-** $Id: qlcdnum.cpp,v 2.0 1996/09/24 10:16:18 agulbra Exp $
+** $Id: qlcdnum.cpp,v 2.2 1997/03/12 15:31:52 paul Exp $
 **
 ** Implementation of QLCDNumber class
 **
 ** Created : 940518
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -20,7 +22,7 @@
 #include "qpainter.h"
 #include <stdio.h>
 
-RCSTAG("$Id: qlcdnum.cpp,v 2.0 1996/09/24 10:16:18 agulbra Exp $");
+RCSTAG("$Id: qlcdnum.cpp,v 2.2 1997/03/12 15:31:52 paul Exp $");
 
 
 /*!
@@ -53,6 +55,9 @@ RCSTAG("$Id: qlcdnum.cpp,v 2.0 1996/09/24 10:16:18 agulbra Exp $");
   the display() slot to another slot as well and store the value
   there.
 
+  \warning The current version of QLCDNumber does not work well
+  with background pixmaps.
+
   Incidentally, QLCDNumber is the very oldest part of Qt, tracing back
   to a BASIC program on the <a
   href=http://www.nvg.unit.no/spectrum/>Sinclair Spectrum</a>.
@@ -242,7 +247,7 @@ static char *getSegments( char ch )		// gets list of segments for ch
 /*!
   Constructs an LCD number, sets the number of digits to 5, the base
   to decimal, the decimal point mode to 'small' and the frame style to
-  a raised box.
+  a raised box. The segmentStyle() is set to \c Outline.
 
   The \e parent and \e name arguments are passed to the QFrame constructor.
 
@@ -260,7 +265,7 @@ QLCDNumber::QLCDNumber( QWidget *parent, const char *name )
 /*!
   Constructs an LCD number, sets the number of digits to \e numDigits, the
   base to decimal, the decimal point mode to 'small' and the frame style
-  to a raised box.
+  to a raised box.  The segmentStyle() is set to \c Outline.
 
   The \e parent and \e name arguments are passed to the QFrame constructor.
 
@@ -286,6 +291,7 @@ void QLCDNumber::init()
     base       = DEC;
     smallPoint = FALSE;
     setNumDigits( ndigits );
+    setSegmentStyle( Outline );
 }
 
 /*!
@@ -764,6 +770,13 @@ void QLCDNumber::drawDigit( const QPoint &pos, QPainter &p, int segLen,
 }
 
 
+static void addPoint( QPointArray &a, const QPoint &p )
+{
+    uint n = a.size();
+    a.resize( n + 1 );
+    a.setPoint( n, p );
+}
+
 /*!
   \internal
 */
@@ -773,21 +786,28 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 {
     QPoint pt = pos;
     QColorGroup g = colorGroup();
-    QColor lightColor,darkColor;
+    QColor lightColor,darkColor,fgColor;
     if ( erase ){
 	lightColor = g.background();
 	darkColor  = g.background();
+	fgColor    = g.background();
     } else {
 	lightColor = g.light();
 	darkColor  = g.dark();
+	fgColor    = g.foreground();
     }
     int width = segLen/5;
 
-#define LINETO(X,Y) p.lineTo(QPoint(pt.x() + (X),pt.y() + (Y)))
-#define LIGHT p.setPen(lightColor)
-#define DARK  p.setPen(darkColor)
+#define LINETO(X,Y) addPoint( a, QPoint(pt.x() + (X),pt.y() + (Y)))
+#define LIGHT 
+#define DARK 
 
-    switch ( segmentNo ) {
+    if ( fill ) {
+	QPointArray a(0);
+
+	//The following is an exact copy of the switch below.
+	//don't make any changes here
+	switch ( segmentNo ) {
 	case 0 :
 	    p.moveTo(pt);
 	    LIGHT;
@@ -909,8 +929,181 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    warning( "QLCDNumber::drawSegment: Internal error."
 		     "  Illegal segment id: %d\n", segmentNo );
 #endif
-    }
+	}
+	// End exact copy
+	p.setPen( fgColor );
+	p.setBrush( fgColor );
+	p.drawPolygon( a );
+	p.setBrush( NoBrush );
+
+	pt = pos;
+    } 
+#undef LINETO
+#undef LIGHT
+#undef DARK
+
+#define LINETO(X,Y) p.lineTo(QPoint(pt.x() + (X),pt.y() + (Y)))
+#define LIGHT p.setPen(lightColor)
+#define DARK  p.setPen(darkColor)
+    if ( shadow )
+	switch ( segmentNo ) {
+	case 0 :
+	    p.moveTo(pt);
+	    LIGHT;
+	    LINETO(segLen - 1,0);
+	    DARK;
+	    LINETO(segLen - width - 1,width);
+	    LINETO(width,width);
+	    LINETO(0,0);
+	    break;
+	case 1 :
+	    pt.ry()++;
+	    p.moveTo(pt);
+	    LIGHT;
+	    LINETO(width,width);
+	    DARK;
+	    LINETO(width,segLen - width/2 - 2);
+	    LINETO(0,segLen - 2);
+	    LIGHT;
+	    LINETO(0,0);
+	    break;
+	case 2 :
+	    pt.rx() += (QCOORD)(segLen - 1);
+	    pt.ry()++;
+	    p.moveTo(pt);
+	    DARK;
+	    LINETO(0,segLen - 2);
+	    LINETO(-width,segLen - width/2 - 2);
+	    LIGHT;
+	    LINETO(-width,width);
+	    LINETO(0,0);
+	    break;
+	case 3 :
+	    pt.ry() += (QCOORD)segLen;
+	    p.moveTo(pt);
+	    LIGHT;
+	    LINETO(width,-width/2);
+	    LINETO(segLen - width - 1,-width/2);
+	    LINETO(segLen - 1,0);
+	    DARK;
+	    if (width & 1) {		// adjust for integer division error
+		LINETO(segLen - width - 3,width/2 + 1);
+		LINETO(width + 2,width/2 + 1);
+	    } else {
+		LINETO(segLen - width - 1,width/2);
+		LINETO(width,width/2);
+	    }
+	    LINETO(0,0);
+	    break;
+	case 4 :
+	    pt.ry() += (QCOORD)(segLen + 1);
+	    p.moveTo(pt);
+	    LIGHT;
+	    LINETO(width,width/2);
+	    DARK;
+	    LINETO(width,segLen - width - 2);
+	    LINETO(0,segLen - 2);
+	    LIGHT;
+	    LINETO(0,0);
+	    break;
+	case 5 :
+	    pt.rx() += (QCOORD)(segLen - 1);
+	    pt.ry() += (QCOORD)(segLen + 1);
+	    p.moveTo(pt);
+	    DARK;
+	    LINETO(0,segLen - 2);
+	    LINETO(-width,segLen - width - 2);
+	    LIGHT;
+	    LINETO(-width,width/2);
+	    LINETO(0,0);
+	    break;
+	case 6 :
+	    pt.ry() += (QCOORD)(segLen*2);
+	    p.moveTo(pt);
+	    LIGHT;
+	    LINETO(width,-width);
+	    LINETO(segLen - width - 1,-width);
+	    LINETO(segLen - 1,0);
+	    DARK;
+	    LINETO(0,0);
+	    break;
+	case 7 :
+	    if ( smallPoint )	// if smallpoint place'.' between other digits
+		pt.rx() += (QCOORD)(segLen + width/2);
+	    else
+		pt.rx() += (QCOORD)(segLen/2);
+	    pt.ry() += (QCOORD)(segLen*2);
+	    p.moveTo(pt);
+	    DARK;
+	    LINETO(width,0);
+	    LINETO(width,-width);
+	    LIGHT;
+	    LINETO(0,-width);
+	    LINETO(0,0);
+	    break;
+	case 8 :
+	    pt.ry() += (QCOORD)(segLen/2 + width);
+	    pt.rx() += (QCOORD)(segLen/2 - width/2 + 1);
+	    p.moveTo(pt);
+	    DARK;
+	    LINETO(width,0);
+	    LINETO(width,-width);
+	    LIGHT;
+	    LINETO(0,-width);
+	    LINETO(0,0);
+	    break;
+	case 9 :
+	    pt.ry() += (QCOORD)(3*segLen/2 + width);
+	    pt.rx() += (QCOORD)(segLen/2 - width/2 + 1);
+	    p.moveTo(pt);
+	    DARK;
+	    LINETO(width,0);
+	    LINETO(width,-width);
+	    LIGHT;
+	    LINETO(0,-width);
+	    LINETO(0,0);
+	    break;
+#if defined(CHECK_RANGE)
+	default :
+	    warning( "QLCDNumber::drawSegment: Internal error."
+		     "  Illegal segment id: %d\n", segmentNo );
+#endif
+	}
+
 #undef LINETO
 #undef LIGHT
 #undef DARK
 }
+
+
+/*!
+  Sets the style of the QLCDNumber to \a s. 
+  <ul>
+  <li>\c Outline gives raised segments filled with the background color. 
+  <li>\c Filled gives raised segments filled with the foreground color.
+  <li>\c Flat gives flat segments filled with the foreground color.
+  
+  \sa segmentStyle()
+*/
+
+void QLCDNumber::setSegmentStyle( SegmentStyle s )
+{
+    fill = ( s == Flat || s == Filled );
+    shadow = ( s == Outline || s == Filled );
+}
+
+
+/*!
+  Returns the style of the QLCDNumber.
+  \sa setSegmentStyle()
+*/
+
+QLCDNumber::SegmentStyle QLCDNumber::segmentStyle() const
+{
+    ASSERT( fill || shadow );
+    if ( !fill && shadow )
+	return Outline;
+    if ( fill && shadow )
+	return Filled;
+    return Flat;
+}
