@@ -1,17 +1,19 @@
 /****************************************************************************
-** $Id: qmenudta.cpp,v 2.0 1996/09/24 10:16:20 agulbra Exp $
+** $Id: qmenudta.cpp,v 2.3 1997/03/18 10:08:46 hanord Exp $
 **
 ** Implementation of QMenuData class
 **
 ** Created : 941128
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -20,7 +22,7 @@
 #include "qpopmenu.h"
 #include "qapp.h"
 
-RCSTAG("$Id: qmenudta.cpp,v 2.0 1996/09/24 10:16:20 agulbra Exp $");
+RCSTAG("$Id: qmenudta.cpp,v 2.3 1997/03/18 10:08:46 hanord Exp $");
 
 
 /*!
@@ -195,15 +197,17 @@ int QMenuData::insertAny( const char *text, const QPixmap *pixmap,
 	if ( popup ) {
 	    popup->selfItem = mi;
 	    menuInsPopup( popup );
-	    QPopupMenu *p = (QPopupMenu*)this;
-	    while ( p && p != popup )
-		p = (QPopupMenu*)p->parentMenu;
-	    if ( p ) {
+	    if ( isPopupMenu ) {		// circular popups?
+		QPopupMenu *p = (QPopupMenu*)this;
+		while ( p && p != popup )
+		    p = (QPopupMenu*)p->parentMenu;
+		if ( p ) {
 #if defined(CHECK_STATE)
-		warning( "QMenuData::insertItem: Circular popup menu ignored");
+		    warning( "QMenuData::insertItem: "
+			     "Circular popup menu ignored");
 #endif
-		delete mi;
-		return 0;
+		    return 0;
+		}
 	    }
 	}
     }
@@ -389,6 +393,12 @@ int QMenuData::insertItem( const QPixmap &pixmap, QPopupMenu *popup,
 /*!
   Inserts a separator at position \e index.
   The separator becomes the last menu item if \e index is negative.
+
+  In a popup menu, a separator is rendered as a horizontal line.  In a
+  Motif menubar, a separator is spacing, so the rest of the items
+  (just "Help", normally) are drawn right-justified.  In a Windows
+  menubar, all separator are ignored (to comply with the Windows style
+  guide).
 */
 
 void QMenuData::insertSeparator( int index )
@@ -637,15 +647,19 @@ QMenuItem *QMenuData::findItem( int id ) const
 	return 0;
     QMenuItemListIt it( *mitems );
     QMenuItem *mi;
-    while ( (mi=it.current()) ) {
-	if ( mi->ident == id )			// this one?
+    while ( (mi=it.current()) ) {		// search this menu
+	++it;
+	if ( mi->ident == id )			// found item
 	    return mi;
-	if ( mi->popup_menu ) {			// recursive search
+    }
+    it.toFirst();
+    while ( (mi=it.current()) ) {		// search submenus
+	++it;
+	if ( mi->popup_menu ) {
 	    mi = mi->popup_menu->findItem( id );
-	    if ( mi )
+	    if ( mi )				// found item
 		return mi;
 	}
-	++it;
     }
     return 0;					// not found
 }
