@@ -1,17 +1,19 @@
 /****************************************************************************
-** $Id: qpopmenu.cpp,v 2.0 1996/09/24 10:16:20 agulbra Exp $
+** $Id: qpopmenu.cpp,v 2.11 1997/03/26 22:21:54 hanord Exp $
 **
 ** Implementation of QPopupMenu class
 **
 ** Created : 941128
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,8 +25,9 @@
 #include "qdrawutl.h"
 #include "qscrbar.h"				// qDrawArrow
 #include "qapp.h"
+#include <ctype.h>
 
-RCSTAG("$Id: qpopmenu.cpp,v 2.0 1996/09/24 10:16:20 agulbra Exp $");
+RCSTAG("$Id: qpopmenu.cpp,v 2.11 1997/03/26 22:21:54 hanord Exp $");
 
 
 // Motif style parameters
@@ -35,8 +38,10 @@ static const int motifSepHeight		= 2;	// separator item height
 static const int motifItemHMargin	= 3;	// menu item hor text margin
 static const int motifItemVMargin	= 2;	// menu item ver text margin
 static const int motifArrowHMargin	= 6;	// arrow horizontal margin
-static const int motifArrowVMargin	= 2;	// arrow horizontal margin
+static const int motifArrowVMargin	= 2;	// arrow vertical margin
 static const int motifTabSpacing	= 12;	// space between text and tab
+static const int motifCheckMarkHMargin	= 2;	// horiz. margins of check mark
+
 
 /*
 
@@ -71,6 +76,174 @@ static const int motifTabSpacing	= 12;	// space between text and tab
 */
 
 
+// size of checkmark image
+
+static void getSizeOfBitmap( int gs, int *w, int *h )
+{
+	if ( gs == WindowsStyle )
+	    *w = *h = 7;
+	else
+	    *w = *h = 6;
+}
+
+
+static int getWidthOfCheckCol( int gs )
+{
+    int w = motifItemFrame + 2 * motifCheckMarkHMargin;
+    if ( gs == WindowsStyle )
+	w += 7;
+    else
+	w += 9;
+    return w;
+}
+
+// Checkmark drawing -- temporarily here...
+static void qDrawCheckMark( QPainter *p, int x, int y, int w, int h,
+			    const QColorGroup &g, GUIStyle gs, bool act )
+{
+    int markW, markH;
+    getSizeOfBitmap( gs, &markW, &markH );
+    int posX = x + ( w - markW )/2;
+    int posY = y + ( h - markH )/2;
+
+    if ( gs == WindowsStyle ) {
+	// Could do with some optimizing/caching...
+	QPointArray a( 7*2 );
+	int i, xx, yy;
+	xx = posX;
+	yy = 3 + posY;
+	for ( i=0; i<3; i++ ) {
+	    a.setPoint( 2*i,   xx, yy );
+	    a.setPoint( 2*i+1, xx, yy+2 );
+	    xx++; yy++;
+	}
+	yy -= 2;
+	for ( i=3; i<7; i++ ) {
+	    a.setPoint( 2*i,   xx, yy );
+	    a.setPoint( 2*i+1, xx, yy+2 );
+	    xx++; yy--;
+	}
+	p->setPen( act ? white : black );
+	p->drawLineSegments( a );
+    }
+    else {
+	QBrush fill( g.mid() );
+	qDrawShadePanel( p, posX, posY, markW, markH, g, TRUE, 2, &fill );
+    }
+}
+
+
+//
+// Creates an accelerator string for the key k.
+// For instance CTRL+Key_O gives "Ctrl+O".
+//
+
+static QString accel_str( int k )
+{
+    QString s;
+    if ( (k & SHIFT) == SHIFT )
+	s = "Shift";
+    if ( (k & CTRL) == CTRL ) {
+	if ( s.isEmpty() )
+	    s = "Ctrl";
+	else
+	    s += "+Ctrl";
+    }
+    if ( (k & ALT) == ALT ) {
+	if ( s.isEmpty() )
+	    s = "Alt";
+	else
+	    s += "+Alt";
+    }
+    k &= ~(SHIFT | CTRL | ALT);
+    QString p;
+    if ( k >= Key_F1 && k <= Key_F24 )
+	p.sprintf( "F%d", k - Key_F1 + 1 );
+    else if ( k >= Key_Space && k <= Key_AsciiTilde )
+	p.sprintf( "%c", k );
+    else {
+	switch ( k ) {
+	    case Key_Escape:
+		p = "Esc";
+		break;
+	    case Key_Tab:
+		p = "Tab";
+		break;
+	    case Key_Backtab:
+		p = "Backtab";
+		break;
+	    case Key_Backspace:
+		p = "Backspace";
+		break;
+	    case Key_Return:
+		p = "Return";
+		break;
+	    case Key_Enter:
+		p = "Enter";
+		break;
+	    case Key_Insert:
+		p = "Ins";
+		break;
+	    case Key_Delete:
+		p = "Del";
+		break;
+	    case Key_Pause:
+		p = "Pause";
+		break;
+	    case Key_Print:
+		p = "Print";
+		break;
+	    case Key_SysReq:
+		p = "SysReq";
+		break;
+	    case Key_Home:
+		p = "Home";
+		break;
+	    case Key_End:
+		p = "End";
+		break;
+	    case Key_Left:
+		p = "Left";
+		break;
+	    case Key_Up:
+		p = "Up";
+		break;
+	    case Key_Right:
+		p = "Right";
+		break;
+	    case Key_Down:
+		p = "Down";
+		break;
+	    case Key_Prior:
+		p = "PgUp";
+		break;
+	    case Key_Next:
+		p = "PgDown";
+		break;
+	    case Key_CapsLock:
+		p = "CapsLock";
+		break;
+	    case Key_NumLock:
+		p = "NumLock";
+		break;
+	    case Key_ScrollLock:
+		p = "ScrollLock";
+		break;
+	    default:
+		p.sprintf( "<%d?>", k );
+		break;
+	}
+    }
+    if ( s.isEmpty() )
+	s = p;
+    else {
+	s += '+';
+	s += p;
+    }
+    return s;
+}
+
+
 /*****************************************************************************
   QPopupMenu member functions
  *****************************************************************************/
@@ -95,16 +268,20 @@ QPopupMenu::QPopupMenu( QWidget *parent, const char *name )
     autoaccel	  = 0;
     accelDisabled = FALSE;
     popupActive	  = -1;
-    tabMark	  = 0;
+    tabCheck	  = 0;
+    setTabMark( 0 );
     setNumCols( 1 );				// set number of table columns
     setNumRows( 0 );				// set number of table rows
     switch ( style() ) {
 	case WindowsStyle:
 	    setFrameStyle( QFrame::WinPanel | QFrame::Raised );
+	    setMouseTracking( TRUE );
+	    setCheckable( TRUE );		
 	    break;
 	case MotifStyle:
 	    setFrameStyle( QFrame::Panel | QFrame::Raised );
 	    setLineWidth( motifPopupFrame );
+	    setCheckable( FALSE );		
 	    break;
 	default:
 	    setFrameStyle( QFrame::Panel | QFrame::Plain );
@@ -126,9 +303,65 @@ QPopupMenu::~QPopupMenu()
 
 void QPopupMenu::updateItem( int id )		// update popup menu item
 {
-    updateCell( indexOf(id), 0, FALSE );
+    updateRow( indexOf(id) );
 }
 
+
+// Double use of tabMark / checkingEnabled until 2.0
+
+/*!
+  Enables or disables display of check marks by the menu items.
+  
+  Notice that checking is always enabled when in windows-style.
+
+  \sa isCheckable(), QMenuData::setItemChecked()
+*/
+
+void QPopupMenu::setCheckable( bool enable )
+{
+    bool oldState = isCheckable();
+    bool newState = (style() == WindowsStyle) || enable;
+    if ( oldState != newState ) {
+	if ( newState ) {
+	    setNumCols( 2 );
+	    tabCheck |= 0x80000000;
+	}
+	else {
+	    setNumCols( 1 );
+	    tabCheck &= 0x7FFFFFFF;
+	}
+	badSize = TRUE;
+	update();
+    }
+}
+
+/*!
+  Returns whether display of check marks by the menu items is enabled.
+
+  \sa setCheckable(), QMenuData::setItemChecked()
+*/
+
+bool QPopupMenu::isCheckable() const
+{
+    return (tabCheck & 0x80000000) != 0;
+}
+
+
+void QPopupMenu::setTabMark( int t )
+{
+    bool e = isCheckable();
+    tabCheck = t;
+    if ( e )
+	tabCheck |= 0x80000000;
+}
+
+
+int QPopupMenu::tabMark()
+{
+    return tabCheck & 0x7FFFFFFF;
+}    
+
+
 void QPopupMenu::menuContentsChanged()
 {
     badSize = TRUE;				// might change the size
@@ -218,8 +451,7 @@ void QPopupMenu::accelActivated( int id )
     if ( mi && mi->isEnabled() ) {
 	if ( mi->signal() )			// activate signal
 	    mi->signal()->activate();
-	else					// normal connection
-	    actSig( mi->id() );
+	actSig( mi->id() );
     }
 }
 
@@ -362,7 +594,11 @@ void QPopupMenu::updateSize()
 {
     int height	  = 0;
     int max_width = 10;
+    GUIStyle gs	  = style();
     QFontMetrics fm = fontMetrics();
+#if 0
+    QFontMetrics fm( font() );
+#endif
     QMenuItemListIt it( *mitems );
     register QMenuItem *mi;
     bool hasSubMenu = FALSE;
@@ -380,15 +616,15 @@ void QPopupMenu::updateSize()
 	} else if ( mi->text() ) {
 	    height += cellh;
 	    const char *s = mi->text();
-	    const char *t = strchr( s, '\t' );
-	    if ( t ) {				// string contains tab
+	    const char *t;
+	    if ( (t=strchr(s, '\t')) ) {	// string contains tab
 		w = fm.width( s, (int)((long)t-(long)s) );
 		int tw = fm.width( t+1 );
 		if ( tw > tab_width )
 		    tab_width = tw;
-	    }
-	    else
+	    } else {
 		w = fm.width( s );
+	    }
 	}
 #if defined(CHECK_NULL)
 	else
@@ -401,11 +637,18 @@ void QPopupMenu::updateSize()
     int extra_width = 0;
     if ( tab_width ) {
 	extra_width = tab_width + motifTabSpacing;
-	tabMark = max_width + motifTabSpacing;
+	setTabMark( max_width + motifTabSpacing );
     }
     else
-	tabMark = 0;
-    max_width  += 2*motifItemHMargin + 2*motifItemFrame;
+	setTabMark( 0 );
+
+    max_width  += 2*motifItemHMargin;
+
+    if ( isCheckable() )
+	max_width += getWidthOfCheckCol( gs ) + motifItemFrame;
+    else
+	max_width += 2*motifItemFrame;
+
     if ( hasSubMenu ) {
 	if ( fm.ascent() + motifArrowHMargin > extra_width )
 	    extra_width = fm.ascent() + motifArrowHMargin;
@@ -417,116 +660,6 @@ void QPopupMenu::updateSize()
 }
 
 
-//
-// Creates an accelerator string for the key k.
-// For instance CTRL+Key_O gives "Ctrl+O".
-//
-
-static QString accel_str( int k )
-{
-    QString s;
-    if ( (k & SHIFT) == SHIFT )
-	s = "Shift";
-    if ( (k & CTRL) == CTRL ) {
-	if ( s.isEmpty() )
-	    s = "Ctrl";
-	else
-	    s += "+Ctrl";
-    }
-    if ( (k & ALT) == ALT ) {
-	if ( s.isEmpty() )
-	    s = "A|t";
-	else
-	    s += "+Alt";
-    }
-    k &= ~(SHIFT | CTRL | ALT);
-    QString p;
-    if ( k >= Key_F1 && k <= Key_F24 )
-	p.sprintf( "F%d", k - Key_F1 + 1 );
-    else if ( k >= Key_Space && k <= Key_AsciiTilde )
-	p.sprintf( "%c", k );
-    else {
-	switch ( k ) {
-	    case Key_Escape:
-		p = "Esc";
-		break;
-	    case Key_Tab:
-		p = "Tab";
-		break;
-	    case Key_Backtab:
-		p = "Backtab";
-		break;
-	    case Key_Backspace:
-		p = "Backspace";
-		break;
-	    case Key_Return:
-		p = "Return";
-		break;
-	    case Key_Enter:
-		p = "Enter";
-		break;
-	    case Key_Insert:
-		p = "Ins";
-		break;
-	    case Key_Delete:
-		p = "Del";
-		break;
-	    case Key_Pause:
-		p = "Pause";
-		break;
-	    case Key_Print:
-		p = "Print";
-		break;
-	    case Key_SysReq:
-		p = "SysReq";
-		break;
-	    case Key_Home:
-		p = "Home";
-		break;
-	    case Key_End:
-		p = "End";
-		break;
-	    case Key_Left:
-		p = "Left";
-		break;
-	    case Key_Up:
-		p = "Up";
-		break;
-	    case Key_Right:
-		p = "Right";
-		break;
-	    case Key_Down:
-		p = "Down";
-		break;
-	    case Key_Prior:
-		p = "PgUp";
-		break;
-	    case Key_Next:
-		p = "PgDown";
-		break;
-	    case Key_CapsLock:
-		p = "CapsLock";
-		break;
-	    case Key_NumLock:
-		p = "NumLock";
-		break;
-	    case Key_ScrollLock:
-		p = "ScrollLock";
-		break;
-	    default:
-		p.sprintf( "<%d?>", k );
-		break;
-	}
-    }
-    if ( s.isEmpty() )
-	s = p;
-    else {
-	s += '+';
-	s += p;
-    }
-    return s;
-}
-
 
 /*!
   \internal
@@ -656,9 +789,18 @@ int QPopupMenu::cellHeight( int row )
     return h;
 }
 
-int QPopupMenu::cellWidth( int )
+
+
+int QPopupMenu::cellWidth( int col )
 {
-    return width() - 2*frameWidth();
+    if ( isCheckable() ) {
+	if ( col == 0 )
+	    return getWidthOfCheckCol(style());
+	else
+	    return width() - ( 2*frameWidth() + getWidthOfCheckCol(style()) );
+    }	    
+    else
+	return width() - 2*frameWidth();	
 }
 
 
@@ -675,27 +817,44 @@ void QPopupMenu::paintCell( QPainter *p, int row, int col )
     if ( !mi->isDirty() )
 	return;
 
-    if ( mi->isSeparator() ) {			// draw separator
-	p->setPen( g.dark() );
-	p->drawLine( 0, 0, cellw, 0 );
-	p->setPen( g.light() );
-	p->drawLine( 0, 1, cellw, 1 );
-	return;
+    int rw = isCheckable() ? totalWidth() : cellw;
+
+    if ( col == 0 ) {
+	if ( mi->isSeparator() ) {			// draw separator
+	    p->setPen( g.dark() );
+	    p->drawLine( 0, 0, rw, 0 );
+	    p->setPen( g.light() );
+	    p->drawLine( 0, 1, rw, 1 );
+	    return;
+	}
+        
+	int pw = motifItemFrame;
+	if ( gs != MotifStyle )
+	    pw = 1;
+	if ( gs == WindowsStyle ) {
+	    p->fillRect( 0, 0, rw, cellh, act ? darkBlue : g.background() );
+	} else if ( gs == MotifStyle ) {
+	    if ( act )				// active item frame
+		qDrawShadePanel( p, 0, 0, rw, cellh, g, FALSE, pw );
+	    else				// incognito frame
+		qDrawPlainRect( p, 0, 0, rw, cellh, g.background(), pw );
+	}
+
+	if ( isCheckable() ) {	// just "checking"...
+	    int mw = cellw - ( 2*motifCheckMarkHMargin + motifItemFrame );
+	    int mh = cellh - 2*motifItemFrame;
+	    if ( mi->isChecked() ) {
+		qDrawCheckMark( p, motifItemFrame + motifCheckMarkHMargin,
+				motifItemFrame, mw, mh, g, gs, act );
+	    }
+	    return;
+	}
     }
-    int pw = motifItemFrame;
-    if ( gs != MotifStyle )
-	pw = 1;
-    if ( gs == WindowsStyle ) {
-	p->fillRect( 0, 0, cellw, cellh, act ? darkBlue : g.background() );
+
+    if ( gs == WindowsStyle )
 	p->setPen( act ? white : g.text() );
-    }
-    else if ( gs == MotifStyle ) {
-	if ( act )				// active item frame
-	    qDrawShadePanel( p, 0, 0, cellw, cellh, g, FALSE, pw );
-	else					// incognito frame
-	    qDrawPlainRect( p, 0, 0, cellw, cellh, g.background(), pw );
+    else
 	p->setPen( g.text() );
-    }
 
     QColor discol;
     if ( dis ) {
@@ -703,31 +862,40 @@ void QPopupMenu::paintCell( QPainter *p, int row, int col )
 	p->setPen( discol );
     }
 
+    int x = motifItemHMargin + ( isCheckable() ? 0 : motifItemFrame);
     if ( mi->pixmap() ) {			// draw pixmap
 	QPixmap *pixmap = mi->pixmap();
 	if ( pixmap->depth() == 1 )
 	    p->setBackgroundMode( OpaqueMode );
-	p->drawPixmap( motifItemFrame + motifItemHMargin, motifItemFrame,
-		       *pixmap );
+	p->drawPixmap( x, motifItemFrame, *pixmap );
 	if ( pixmap->depth() == 1 )
 	    p->setBackgroundMode( TransparentMode );
     } else if ( mi->text() ) {			// draw text
 	const char *s = mi->text();
 	const char *t = strchr( s, '\t' );
-	int x = motifItemFrame + motifItemHMargin;
 	int m = motifItemVMargin;
 	const int text_flags = AlignVCenter|ShowPrefix | DontClip | SingleLine;
 	if ( t ) {				// draw text before tab
+	    if ( gs == WindowsStyle && dis && !act ) {
+		p->setPen( white );
+		p->drawText( x+1, m+1, cellw, cellh-2*m, text_flags,
+			     s, (int)((long)t-(long)s) );
+		p->setPen( discol );
+	    }
 	    p->drawText( x, m, cellw, cellh-2*m, text_flags,
 			 s, (int)((long)t-(long)s) );
 	    s = t + 1;
-	    x = tabMark;
+	    x = tabMark();
+	}
+	if ( gs == WindowsStyle && dis && !act ) {
+	    p->setPen( white );
+	    p->drawText( x+1, m+1, cellw, cellh-2*m, text_flags, s );
+	    p->setPen( discol );
 	}
 	p->drawText( x, m, cellw, cellh-2*m, text_flags, s );
     }
     if ( mi->popup() ) {			// draw sub menu arrow
 	int dim = (cellh-2*motifItemFrame) / 2;
-	dim /= 2;
 	if ( gs == WindowsStyle && row == actItem ) {
 	    if ( !dis )
 		discol = white;
@@ -750,7 +918,7 @@ void QPopupMenu::paintAll()
     while ( (mi=it.current()) ) {
 	++it;
 	if ( mi->isDirty() )			// this item needs a refresh
-	    updateCell( row, 0, FALSE );
+	    updateRow( row );
 	++row;
     }
 }
@@ -781,7 +949,7 @@ void QPopupMenu::mousePressEvent( QMouseEvent *e )
     mouseBtDn = TRUE;				// mouse button down
     int item = itemAtPos( e->pos() );
     if ( item == -1 ) {
-	if ( !rect().contains( e->pos() ) && !tryMenuBar( e ) ) {
+	if ( !rect().contains(e->pos()) && !tryMenuBar(e) ) {
 	    hide();
 	    byeMenuBar();
 	}
@@ -791,8 +959,7 @@ void QPopupMenu::mousePressEvent( QMouseEvent *e )
     if ( item != actItem ) {			// new item activated
 	actItem = item;
 	repaint( FALSE );
-	if ( mi->id() >= 0 )
-	    hilitSig( mi->id() );
+	hilitSig( mi->id() );
     }
     QPopupMenu *popup = mi->popup();
     if ( popup ) {
@@ -800,15 +967,14 @@ void QPopupMenu::mousePressEvent( QMouseEvent *e )
 	    popup->actItem = -1;
 	    popup->hidePopups();
 	    popup->repaint( FALSE );
-	}
-	else {					// open sub menu
+	} else {				// open sub menu
 	    hidePopups();
 	    killTimers();
 	    startTimer( 20 );
 	}
-    }
-    else
+    } else {
 	hidePopups();
+    }
 }
 
 /*!
@@ -836,12 +1002,10 @@ void QPopupMenu::mouseReleaseEvent( QMouseEvent *e )
 	    if ( mi->isEnabled() ) {
 		if ( mi->signal() )		// activate signal
 		    mi->signal()->activate();
-		else				// normal connection
-		    actSig( mi->id() );
+		actSig( mi->id() );
 	    }
 	}
-    }
-    else {
+    } else {
 	hideAllPopups();
 	byeMenuBar();
     }
@@ -853,18 +1017,19 @@ void QPopupMenu::mouseReleaseEvent( QMouseEvent *e )
 
 void QPopupMenu::mouseMoveEvent( QMouseEvent *e )
 {
+    if ( (e->state() & MouseButtonMask) == 0 && !hasMouseTracking() )
+	return;
     int	 item = itemAtPos( e->pos() );
     if ( item == -1 ) {				// no valid item
 	if ( popupActive == -1 ) {		// no active popup sub menu
 	    int lastActItem = actItem;
 	    actItem = -1;
 	    if ( lastActItem >= 0 )
-		updateCell( lastActItem, 0, FALSE );
+		updateRow( lastActItem );
 	}
 	if ( !rect().contains( e->pos() ) && !tryMenuBar( e ) )
 	    hidePopups();
-    }
-    else {					// mouse on valid item
+    } else {					// mouse on valid item
 	register QMenuItem *mi = mitems->at( item );
 	QPopupMenu *popup = mi->popup();
 	if ( actItem == item ) {
@@ -883,10 +1048,9 @@ void QPopupMenu::mouseMoveEvent( QMouseEvent *e )
 	}
 	hidePopups();				// hide popup items
 	if ( lastActItem >= 0 )
-	    updateCell( lastActItem, 0, FALSE );
-	updateCell( actItem, 0, FALSE );
-	if ( mi->id() >= 0 )			// valid identifier
-	    hilitSig( mi->id() );
+	    updateRow( lastActItem );
+	updateRow( actItem );
+	hilitSig( mi->id() );
     }
 }
 
@@ -906,48 +1070,77 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
     bool ok_key = TRUE;
 
     switch ( e->key() ) {
-	case Key_Up:
-	    d = -1;
-	    break;
+    case Key_Up:
+	d = -1;
+	break;
 
-	case Key_Down:
-	    d = 1;
-	    break;
+    case Key_Down:
+	d = 1;
+	break;
 
-	case Key_Escape:
+    case Key_Escape:
+	hideAllPopups();
+	byeMenuBar();
+	break;
+
+    case Key_Return:
+    case Key_Enter:
+	if ( actItem < 0 )
+	    break;
+	mi = mitems->at( actItem );
+	popup = mi->popup();
+	if ( popup ) {
+	    hidePopups();
+	    killTimers();
+	    startTimer( 20 );
+	    popup->setFirstItemActive();
+	} else {
 	    hideAllPopups();
 	    byeMenuBar();
-	    break;
-
-	case Key_Return:
-	case Key_Enter:
-	    if ( actItem < 0 )
-		break;
-	    mi = mitems->at( actItem );
-	    popup = mi->popup();
-	    if ( popup ) {
-		hidePopups();
-		killTimers();
-		startTimer( 20 );
-		popup->setFirstItemActive();
+	    if ( mi->isEnabled() ) {
+		if ( mi->signal() )
+		    mi->signal()->activate();
+		actSig( mi->id() );
 	    }
-	    else {
-		hideAllPopups();
-		byeMenuBar();
-		if ( mi->isEnabled() ) {
-		    if ( mi->signal() )
-			mi->signal()->activate();
-		    else
-			actSig( mi->id() );
-		}
-	    }
-	    break;
+	}
+	break;
 
-	default:
-	    ok_key = FALSE;
+    default:
+	ok_key = FALSE;
 
     }
+#if 1
+    if ( !ok_key && !e->state() && e->key() >= Key_A && e->key() <= Key_Z ) {
+	char c = 'A' + e->key() - Key_A;
 
+	QMenuItemListIt it(*mitems);
+	register QMenuItem *m;
+	while ( (m=it.current()) ) {
+	    ++it;
+	    QString s = m->text();
+	    if ( !s.isEmpty() ) {
+		int i = s.find( '&' );
+		if ( i >= 0 && isalnum(s[i+1]) ) {
+		    int k = s[i+1];
+		    if ( toupper(k) == c ) {
+			mi = m;
+			ok_key = TRUE;
+			break;
+		    }
+		}
+	    }
+	}
+	if ( mi ) {
+	    hideAllPopups();
+	    byeMenuBar();
+	    if ( mi->isEnabled() ) {
+		if ( mi->signal() )
+		    mi->signal()->activate();
+		actSig( mi->id() );
+	    }
+	}
+    }
+#endif
     if ( !ok_key ) {				// send to menu bar
 	register QMenuData *top = this;		// find top level
 	while ( top->parentMenu )
@@ -975,8 +1168,8 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 	    actItem = i;
 	    if ( mi->id() >= 0 )
 		hilitSig( mi->id() );
-	    updateCell( lastActItem, 0, FALSE );
-	    updateCell( actItem, 0, FALSE );
+	    updateRow( lastActItem );
+	    updateRow( actItem );
 	}
     }
 }
@@ -1001,3 +1194,11 @@ void QPopupMenu::timerEvent( QTimerEvent *e )
 	popup->popup( mapToGlobal(pos) );
     }
 }
+
+
+void QPopupMenu::updateRow( int row )
+{
+    updateCell( row, 0, FALSE );
+    if ( isCheckable() )
+	updateCell( row, 1, FALSE );
+}
