@@ -1,17 +1,19 @@
 /**********************************************************************
-** $Id: qtablevw.cpp,v 2.0 1996/09/24 10:16:22 agulbra Exp $
+** $Id: qtablevw.cpp,v 2.18 1997/03/16 13:36:29 paul Exp $
 **
 ** Implementation of QTableView class
 **
 ** Created : 941115
 **
-** Copyright (C) 1992-1996 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.0
-** see the file LICENSE included in the distribution for the usage
+** This file is part of the non-commercial distribution of Qt 1.2.
+**
+** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the commercial version of Qt only by this comment.)
+** (This file differs from the one in the commercial distribution of
+** Qt only by this comment.)
 **
 *****************************************************************************/
 
@@ -19,8 +21,9 @@
 #include "qscrbar.h"
 #include "qpainter.h"
 #include "qdrawutl.h"
+#include <limits.h>
 
-RCSTAG("$Id: qtablevw.cpp,v 2.0 1996/09/24 10:16:22 agulbra Exp $");
+RCSTAG("$Id: qtablevw.cpp,v 2.18 1997/03/16 13:36:29 paul Exp $");
 
 
 const int sbDim = 16;
@@ -70,7 +73,9 @@ void CornerSquare::paintEvent( QPaintEvent * )
   \ingroup abstractwidgets
 
   A table view consists of a number of abstract cells organized in rows
-  and columns and a visible part called a view.
+  and columns and a visible part called a view. The cells are identified
+  with a row index and a column index. The top left cell is in row 0,
+  column 0.
 
   The behavior of the widget can be finely tuned using
   setTableFlags(); subclasses like QListBox are little more than a
@@ -83,25 +88,25 @@ void CornerSquare::paintEvent( QPaintEvent * )
   distinctions between the four coordinate systems involved.
 
   <ol>
-  <li> The cell coordinates.  (0,0) is the top left corner of a cell.
+  <li> The \e cell coordinates.  (0,0) is the top left corner of a cell.
   This is used by functions such as paintCell().
 
-  <li> The table coordinates.  (0,0) is the top left corner of the cell at
-  row 0 and column 0, and uses absolute x and y coordinates.  This is used
-  by functions such as setXOffset() or maxYOffset().
+  <li> The \e table coordinates.  (0,0) is the top left corner of the cell at
+  row 0 and column 0. These coordinates are absolute; that is, they are
+  independent of what part of the table is visible at the moment. This is
+  used by functions such as setXOffset() or maxYOffset().
 
-  <li> The widget coordinates.	(0,0) is the top left corner of the
-  widget, \e including the frame.  This is used by functions such as
-  repaint().
+  <li> The \e widget coordinates. (0,0) is the top left corner of the widget,
+  \e including the frame.  This is used by functions such as repaint().
 
-  <li> The view coordinates.  (0,0) is the top left corner of the
-  view, \e excluding the frame.	 This is the least-used coordinate
-  system, used by functions such as viewWidth().
-  </ol>
+  <li> The \e view coordinates.  (0,0) is the top left corner of the view, \e
+  excluding the frame.  This is the least-used coordinate system, used by
+  functions such as viewWidth().  </ol>
 
   It is rather unfortunate that we have to use four different
   coordinate systems, but if we were to provide a flexible and
-  powerful base class, there wasn't any way around it.
+  powerful base class, there wasn't any way around it.  
+
 */
 
 
@@ -118,6 +123,11 @@ void CornerSquare::paintEvent( QPaintEvent * )
 
   Frame line shapes (QFrame::HLink and QFrame::VLine) are disallowed,
   see QFrame::setStyle().
+
+  Note that the \a f argument is \e not \link setTableFlags() table
+  flags \endlink but rather \link QWidget::QWidget() widget
+  flags. \endlink
+
 */
 
 QTableView::QTableView( QWidget *parent, const char *name, WFlags f )
@@ -131,7 +141,6 @@ QTableView::QTableView( QWidget *parent, const char *name, WFlags f )
     xOffs		 = yOffs      = 0;	// zero total pixel offset
     cellH		 = cellW      = 0;	// user defined cell size
     tFlags		 = 0;
-    doUpdate		 = TRUE;
     vScrollBar		 = hScrollBar = 0;	// no scroll bars
     cornerSquare	 = 0;
     sbDirty		 = 0;
@@ -157,20 +166,14 @@ QTableView::~QTableView()
 
 
 /*!
-  Reimplements QWidget::setBackgroundColor().
-  Sets the background color for the scroll bars, too.
+  \internal
+  Reimplements QWidget::setBackgroundColor() for binary compatibility.
   \sa setPalette()
 */
 
 void QTableView::setBackgroundColor( const QColor &c )
 {
     QWidget::setBackgroundColor( c );
-    if ( cornerSquare )
-	cornerSquare->setBackgroundColor( c );
-    if ( vScrollBar )
-	vScrollBar->setBackgroundColor( c );
-    if ( hScrollBar )
-	hScrollBar->setBackgroundColor( c );
 }
 
 /*!
@@ -210,7 +213,8 @@ void QTableView::show()
   Repaints the table view directly by calling paintEvent() directly,
   unless updates are disabled.
 
-  Erases the view area \e (x,y,w,h) if \e erase is TRUE.
+  Erases the view area \e (x,y,w,h) if \e erase is TRUE. Parameters \e
+  (x,y) are in \e widget coordinates.
 
   If \e w is negative, it is replaced with <code>width() - x</code>.
   If \e h is negative, it is replaced width <code>height() - y</code>.
@@ -222,8 +226,7 @@ void QTableView::show()
   At present, QTableView is the only widget that reimplements \link
   QWidget::repaint() repaint()\endlink.	 It does this because by
   clearing and then repainting one cell at at time, it can make the
-  screen flicker less than it would otherwise.
-*/
+  screen flicker less than it would otherwise.  */
 
 void QTableView::repaint( int x, int y, int w, int h, bool erase )
 {
@@ -316,14 +319,14 @@ void QTableView::setNumCols( int cols )
 
 /*!
   \fn int QTableView::topCell() const
-  Returns the index of the first line in the table that is visible in
-  the view.  The index of the very first line is 0.
+  Returns the index of the first row in the table that is visible in
+  the view.  The index of the very first row is 0.
   \sa leftCell(), setTopCell()
 */
 
 /*!
   Scrolls the table such that \e row becomes the top row.
-  The index of the very first line is 0.
+  The index of the very first row is 0.
   \sa setYOffset(), setTopLeftCell(), setLeftCell()
 */
 
@@ -393,14 +396,14 @@ void QTableView::setTopLeftCell( int row, int col )
 /*!
   \fn int QTableView::xOffset() const
 
-  Returns the coordinate in pixels of the table point which is
+  Returns the x coordinate in \e table coordinates of the pixel which is
   currently on the left edge of the view.
 
-  \sa setXOffset(), yOffset(), leftCell()
-*/
+  \sa setXOffset(), yOffset(), leftCell() */
 
 /*!
   Scrolls the table such that \e x becomes the leftmost pixel in the view.
+  The \e x parameter is in \e table coordinates.
 
   The interaction with \link setTableFlags() Tbl_snapToHGrid
   \endlink is tricky.  If \e updateScrBars is TRUE, the scroll bars are
@@ -417,14 +420,16 @@ void QTableView::setXOffset( int x )
 /*!
   \fn int QTableView::yOffset() const
 
-  Returns the coordinate in pixels of the table point which is currently on
-  the top edge of the view.
+  Returns the y coordinate in \e table coordinates of the pixel which is
+  currently on the top edge of the view.
+
   \sa setYOffset(), xOffset(), topCell()
 */
 
 
 /*!
   Scrolls the table such that \e y becomes the top pixel in the view.
+  The \e y parameter is in \e table coordinates.
 
   The interaction with \link setTableFlags() Tbl_snapToVGrid
   \endlink is tricky.  If \e updateScrBars is TRUE, the scroll bars are
@@ -440,7 +445,7 @@ void QTableView::setYOffset( int y )
 
 /*!
   Scrolls the table such that \e (x,y) becomes the top left pixel
-  in the view.
+  in the view. Parameters \e (x,y) are in \e table coordinates.
 
   The interaction with \link setTableFlags() Tbl_snapTo*Grid \endlink
   is tricky.  If \e updateScrBars is TRUE, the scroll bars are
@@ -472,8 +477,8 @@ void QTableView::setOffset( int x, int y, bool updateScrBars )
 	    xCellDelta	= 0;
 	}
     } else {
-	int xn=0, xcd, col = 0;
-	while ( col < nCols && x > xn+(xcd=cellWidth(col)) ) {
+	int xn=0, xcd=0, col = 0;
+	while ( col < nCols && x >= xn+(xcd=cellWidth(col)) ) {
 	    xn += xcd;
 	    col++;
 	}
@@ -497,13 +502,13 @@ void QTableView::setOffset( int x, int y, bool updateScrBars )
 	}
 	yCellDelta  = (short)(y % cellH);
     } else {
-	int yn=0, yrd, row=0;
-	while ( row < nRows && y > yn+(yrd=cellHeight(row)) ) {
+	int yn=0, yrd=0, row=0;
+	while ( row < nRows && y >= yn+(yrd=cellHeight(row)) ) {
 	    yn += yrd;
 	    row++;
 	}
 	yCellOffs = row;
-	if ( testTableFlags(Tbl_snapToHGrid) ) {
+	if ( testTableFlags(Tbl_snapToVGrid) ) {
 	    yCellDelta = 0;
 	    y = yn;
 	} else {
@@ -560,8 +565,8 @@ void QTableView::setCellWidth( int cellWidth )
     if ( cellW == cellWidth )
 	return;
 #if defined(CHECK_RANGE)
-    if ( cellWidth < 0 ) {
-	warning( "QTableView::setCellWidth: Negative argument" );
+    if ( cellWidth < 0 || cellWidth > SHRT_MAX ) {
+	warning( "QTableView::setCellWidth: Argument out of range" );
 	return;
     }
 #endif
@@ -610,8 +615,8 @@ void QTableView::setCellHeight( int cellHeight )
     if ( cellH == cellHeight )
 	return;
 #if defined(CHECK_RANGE)
-    if ( cellHeight < 0 ) {
-	warning( "QTableView::setCellHeight: Negative argument" );
+    if ( cellHeight < 0 || cellHeight > SHRT_MAX ) {
+	warning( "QTableView::setCellHeight: Argument out of range" );
 	return;
     }
 #endif
@@ -629,11 +634,9 @@ void QTableView::setCellHeight( int cellHeight )
   have variable cell widths and a non-trivial cellWidth() function, or a
   large number of columns in the table.
 
-  The function (or rather, its default implementation) is rather slow
-  for very wide tables.
+  The default implementation may be slow for very wide tables.
 
-  \sa cellWidth(), totalHeight()
-*/
+  \sa cellWidth(), totalHeight() */
 
 int QTableView::totalWidth()
 {
@@ -654,8 +657,7 @@ int QTableView::totalWidth()
   have variable cell heights and a non-trivial cellHeight() function, or a
   large number of rows in the table.
 
-  The function (or rather, its default implementation) is rather slow
-  for very tall tables.
+  The default implementation may be slow for very tall tables.
 
   \sa cellHeight(), totalWidth()
 */
@@ -736,7 +738,7 @@ int QTableView::totalHeight()
   <dt> Tbl_snapToHGrid <dd> Except when the user is actually scrolling,
   the leftmost column shown snaps to the leftmost edge of the view.
   <dt> Tbl_snapToVGrid <dd> Except when the user is actually
-  scrolling, the top line snaps to the top edge of the view.
+  scrolling, the top row snaps to the top edge of the view.
   <dt> Tbl_snapToGrid <dd> The union of the previous two flags.
   </dl>
 
@@ -906,11 +908,14 @@ void QTableView::clearTableFlags( uint f )
 
 void QTableView::setAutoUpdate( bool enable )
 {
-    if ( (bool)doUpdate == enable )
+    obsoleteUpd = enable; //### binary compatibility contortions
+    if ( isUpdatesEnabled() == enable )
 	return;
-    doUpdate = enable;
-    if ( doUpdate )
+    setUpdatesEnabled( enable );
+    if ( enable ) {
 	showOrHideScrollBars();
+	updateScrollBars();
+    }
 }
 
 
@@ -940,14 +945,16 @@ void QTableView::updateCell( int row, int col, bool erase )
 /*!
   \fn QRect QTableView::cellUpdateRect() const
 
-  This function should only be called from the updateCell() function
-  in subclasses. It returns the portion of a cell that actually needs
-  to be updated. This is only useful for non-trivial updateCell().
+  This function should only be called from the paintCell() function in
+  subclasses. It returns the portion of a cell that actually needs to be
+  updated, in \e cell coordinates. This is only useful for non-trivial
+  paintCell().  
+
 */
 
 /*!
   Returns the rectangle which is the actual table, excluding any
-  frame, in the widget coordinate system.
+  frame, in \e widget coordinates.
 
   Somewhat similar to clientRect(), but does not include any frames.
 */
@@ -1222,16 +1229,16 @@ void QTableView::setupPainter( QPainter * )
   (row,col) using \e p, which is open when paintCell() is called and
   must remain open.
 
-  The coordinate system is \link QPainter::translate() translated
-  \endlink such that the origin is at the top left corner of the cell
-  to be painted.  Do not scale or shear the coordinate system (or if
-  you do, restore the transformation matrix before you return).
+  The coordinate system is \link QPainter::translate() translated \endlink
+  such that the origin is at the top left corner of the cell to be
+  painted; i.e. \e cell coordinates.  Do not scale or shear the coordinate
+  system (or if you do, restore the transformation matrix before you
+  return).
 
-  By default, the painter is not clipped, which is probably not what
-  you want.  Call setTableFlags(Tbl_clipCellPainting) to enable clipping.
+  By default, the painter is not clipped, for maximum efficiency. For safety,
+  call setTableFlags(Tbl_clipCellPainting) to enable clipping.
 
-  \sa paintEvent(), QPainter(), setTableFlags()
-*/
+  \sa paintEvent(), QPainter(), setTableFlags() */
 
 
 /*!
@@ -1310,7 +1317,8 @@ void QTableView::paintEvent( QPaintEvent *e )
 	    cellR.setRect( xPos, yPos, cellW ? cellW : cellWidth(col),
 				       cellH ? cellH : cellHeight(row) );
 	    cellUR = cellR.intersect( updateR );
-	    cellUpdateR.setRect( 0, 0, cellUR.width(), cellUR.height() );
+	    cellUpdateR = cellUR;
+	    cellUpdateR.moveBy( -xPos, -yPos ); // cell coordinates
 	    if ( eraseInPaint )
 		paint.eraseRect( cellUR );
 
@@ -1395,15 +1403,19 @@ const QScrollBar *QTableView::verticalScrollBar() const
 {
     QTableView *that = (QTableView*)this; // semantic const
     if ( !vScrollBar ) {
-	that->vScrollBar = new QScrollBar( QScrollBar::Vertical, that );
-	that->vScrollBar->setTracking( FALSE );
-	connect( vScrollBar, SIGNAL(valueChanged(int)),
+	QScrollBar *sb = new QScrollBar( QScrollBar::Vertical, that );
+	CHECK_PTR(sb);
+	sb->setTracking( FALSE );
+	sb->setFocusPolicy( NoFocus );
+	connect( sb, SIGNAL(valueChanged(int)),
 		 SLOT(verSbValue(int)));
-	connect( vScrollBar, SIGNAL(sliderMoved(int)),
+	connect( sb, SIGNAL(sliderMoved(int)),
 		 SLOT(verSbSliding(int)));
-	connect( vScrollBar, SIGNAL(sliderReleased()),
+	connect( sb, SIGNAL(sliderReleased()),
 		 SLOT(verSbSlidingDone()));
-	that->vScrollBar->hide();
+	sb->hide();
+	that->vScrollBar = sb;
+	return sb;
     }
     return vScrollBar;
 }
@@ -1418,15 +1430,19 @@ const QScrollBar *QTableView::horizontalScrollBar() const
 {
     QTableView *that = (QTableView*)this; // semantic const
     if ( !hScrollBar ) {
-	that->hScrollBar = new QScrollBar( QScrollBar::Horizontal, that );
-	that->hScrollBar->setTracking( FALSE );
-	connect( hScrollBar, SIGNAL(valueChanged(int)),
+	QScrollBar *sb = new QScrollBar( QScrollBar::Horizontal, that );
+	sb->setFocusPolicy( NoFocus );
+	CHECK_PTR(sb);
+	sb->setTracking( FALSE );
+	connect( sb, SIGNAL(valueChanged(int)),
 		 SLOT(horSbValue(int)));
-	connect( hScrollBar, SIGNAL(sliderMoved(int)),
+	connect( sb, SIGNAL(sliderMoved(int)),
 		 SLOT(horSbSliding(int)));
-	connect( hScrollBar, SIGNAL(sliderReleased()),
+	connect( sb, SIGNAL(sliderReleased()),
 		 SLOT(horSbSlidingDone()));
-	that->hScrollBar->hide();
+	sb->hide();
+	that->hScrollBar = sb;
+	return sb;
     }
     return hScrollBar;
 }
@@ -1442,6 +1458,7 @@ void QTableView::setHorScrollBar( bool on, bool update )
 	tFlags |= Tbl_hScrollBar;
 	if ( !hScrollBar ) {
 	    hScrollBar = new QScrollBar( QScrollBar::Horizontal, this );
+	    hScrollBar->setFocusPolicy( NoFocus );
 	    hScrollBar->setTracking( FALSE );
 	    connect( hScrollBar, SIGNAL(valueChanged(int)),
 		     SLOT(horSbValue(int)));
@@ -1494,6 +1511,7 @@ void QTableView::setVerScrollBar( bool on, bool update )
 	if ( !vScrollBar ) {
 	    vScrollBar = new QScrollBar( QScrollBar::Vertical, this );
 	    vScrollBar->setTracking( FALSE );
+	    vScrollBar->setFocusPolicy( NoFocus );
 	    connect( vScrollBar, SIGNAL(valueChanged(int)),
 		     SLOT(verSbValue(int)));
 	    connect( vScrollBar, SIGNAL(sliderMoved(int)),
@@ -1626,11 +1644,12 @@ int QTableView::findRawCol( int xPos, int *cellMaxX, int *cellMinX ,
 }
 
 
-/*!
-  Returns the index of the row at position \e yPos.
-  Returns -1 if \e yPos is outside the valid range.
+/*!  
+  Returns the index of the row at position \e yPos, where \e yPos is in
+  \e widget coordinates.  Returns -1 if \e yPos is outside the valid
+  range.
 
-  \sa findCol(), rowYPos()
+  \sa findCol(), rowYPos() 
 */
 
 int QTableView::findRow( int yPos ) const
@@ -1645,11 +1664,12 @@ int QTableView::findRow( int yPos ) const
 }
 
 
-/*!
-  Returns the index of the column at position \e xPos.
-  Returns -1 if \e xPos is outside the valid range.
+/*!  
+  Returns the index of the column at position \e xPos, where \e xPos is
+  in \e widget coordinates.  Returns -1 if \e xPos is outside the valid
+  range.
 
-  \sa findRow(), colXPos()
+  \sa findRow(), colXPos() 
 */
 
 int QTableView::findCol( int xPos ) const
@@ -1667,11 +1687,11 @@ int QTableView::findCol( int xPos ) const
 /*!
   Computes the position in the widget of row \e row.
 
-  Returns TRUE and stores the result in \e *yPos if the row is
-  visible.  Returns FALSE and does not modify \e *yPos if \e row is
-  invisible or invalid.
+  Returns TRUE and stores the result in \e *yPos (in \e widget
+  coordinates) if the row is visible.  Returns FALSE and does not modify
+  \e *yPos if \e row is invisible or invalid.
 
-  \sa colXPos(), findRow()
+  \sa colXPos(), findRow() 
 */
 
 bool QTableView::rowYPos( int row, int *yPos ) const
@@ -1706,11 +1726,11 @@ bool QTableView::rowYPos( int row, int *yPos ) const
 /*!
   Computes the position in the widget of column \e column.
 
-  Returns TRUE and stores the result in \e *xPos if the column is
-  visible.  Returns FALSE and does not modify \e *xPos if \e col is
-  invisible or invalid.
+  Returns TRUE and stores the result in \e *xPos (in \e widget
+  coordinates) if the column is visible.  Returns FALSE and does not
+  modify \e *xPos if \e col is invisible or invalid.
 
-  \sa rowYPos(), findCol()
+  \sa rowYPos(), findCol() 
 */
 
 bool QTableView::colXPos( int col, int *xPos ) const
@@ -1776,7 +1796,10 @@ void QTableView::scroll( int xPixels, int yPixels )
 	// have to erase an area no longer used for
 	// cell drawing.
 
-	int oldLim = 0, newLim = 0; // both =0 just to get around -Wunused
+	int oldLim;
+#if 0
+	int newLim;
+#endif
 	if ( testTableFlags(Tbl_cutCellsH) ) {
 	    int maxX = maxViewX();
 	    int oldLastMaxX, oldLastMinX;
@@ -1787,7 +1810,9 @@ void QTableView::scroll( int xPixels, int yPixels )
 	    oldLastMaxX += xPixels;
 	    oldLastMinX += xPixels;
 	    oldLim = oldLastMaxX <= maxX ? oldLastMaxX : oldLastMinX - 1;
+#if 0
 	    newLim = newLastMaxX <= maxX ? newLastMaxX : newLastMinX - 1;
+#endif
 	    width -= maxX - oldLim;
 
 	    bitBlt( this, xStart - xPixels, yStart,
@@ -1821,7 +1846,10 @@ void QTableView::scroll( int xPixels, int yPixels )
       // have to erase an area no longer used for
       // cell drawing.
 
-	int oldLim = 0, newLim = 0; // both =0 just to get around -Wunused
+	int oldLim;
+#if 0
+	int newLim;
+#endif
 	if ( testTableFlags(Tbl_cutCellsV) ) {
 	    int maxY = maxViewY();
 	    int oldLastMaxY, oldLastMinY;
@@ -1832,7 +1860,9 @@ void QTableView::scroll( int xPixels, int yPixels )
 	    oldLastMaxY += yPixels;
 	    oldLastMinY += yPixels;
 	    oldLim = oldLastMaxY <= maxY ? oldLastMaxY : oldLastMinY - 1;
+#if 0
 	    newLim = newLastMaxY <= maxY ? newLastMaxY : newLastMinY - 1;
+#endif
 	    height -= maxY - oldLim;
 
 	    bitBlt( this, xStart, yStart - yPixels,
@@ -1859,7 +1889,7 @@ void QTableView::scroll( int xPixels, int yPixels )
 
 
 /*!
-  Returns the rightmost pixel of the table view in screen
+  Returns the rightmost pixel of the table view in \e view
   coordinates.	This excludes the frame and any scroll bar, but
   includes blank pixels to the right of the visible table data.
 
@@ -1874,7 +1904,7 @@ int QTableView::maxViewX() const
 
 
 /*!
-  Returns the bottom pixel of the table view in screen
+  Returns the bottom pixel of the table view in \e view
   coordinates.	This excludes the frame and any scroll bar, but
   includes blank pixels below the visible table data.
 
@@ -1888,12 +1918,12 @@ int QTableView::maxViewY() const
 }
 
 
-/*!
-  Returns the width of the table view, as such.	 This does not
-  include any scroll bar or frame, but does include background pixels
-  to the right of the table data.
+/*!  
+  Returns the width of the table view, as such, in \e view coordinates.
+  This does not include any scroll bar or frame, but does include
+  background pixels to the right of the table data.
 
-  \sa maxViewX(), viewHeight(), contentsRect()
+  \sa maxViewX(), viewHeight(), contentsRect() 
 */
 
 int QTableView::viewWidth() const
@@ -1902,11 +1932,12 @@ int QTableView::viewWidth() const
 }
 
 
-/*!
-  Returns the height of the table view, as such.  This does not
-  include any scroll bar or frame, but does include background pixels
-  below the table data.
-  \sa maxViewY() viewWidth() contentsRect()
+/*!  
+  Returns the height of the table view, as such, in \e view
+  coordinates.  This does not include any scroll bar or frame, but does
+  include background pixels below the table data.  
+
+  \sa maxViewY() viewWidth() contentsRect() 
 */
 
 int QTableView::viewHeight() const
@@ -2001,7 +2032,7 @@ void QTableView::updateScrollBars( uint f )
     if ( testTableFlags(Tbl_hScrollBar) && (sbDirty & horMask) != 0 ) {
 	if ( sbDirty & horGeometry )
 	    hScrollBar->setGeometry( 0,height() - sbDim,
-				     viewWidth() + frameWidth()*2, sbDim );
+				     viewWidth() + frameWidth()*2+1, sbDim );
 
 	if ( sbDirty & horSteps ) {
 	    if ( cellW )
@@ -2024,7 +2055,7 @@ void QTableView::updateScrollBars( uint f )
     if ( testTableFlags(Tbl_vScrollBar) && (sbDirty & verMask) != 0 ) {
 	if ( sbDirty & verGeometry )
 	    vScrollBar->setGeometry( width() - sbDim, 0,
-				     sbDim, viewHeight() + frameWidth()*2 );
+				     sbDim, viewHeight() + frameWidth()*2+1 );
 
 	if ( sbDirty & verSteps ) {
 	    if ( cellH )
@@ -2073,7 +2104,7 @@ void QTableView::updateFrameSize()
 
 /*!
   Returns the maximum horizontal offset within the table of the
-  view's left edge.
+  view's left edge, in \e table coordinates.
 
   This is used mainly to set the horizontal scroll bar's range.
 
@@ -2116,7 +2147,7 @@ int QTableView::maxXOffset()
 
 /*!
   Returns the maximum horizontal offset within the table of the
-  view's top edge.
+  view's top edge, in \e table coordinates.
 
   This is used mainly to set the vertical scroll bar's range.
 
@@ -2158,7 +2189,7 @@ int QTableView::maxYOffset()
 
 
 /*!
-  Returns the idex of the last column which may be at the left edge
+  Returns the index of the last column which may be at the left edge
   of the view.
 
   Depending on the \link setTableFlags Tbl_scrollLastHCell\endlink flag,
@@ -2173,7 +2204,7 @@ int QTableView::maxColOffset()
     if ( cellW )
 	return mx/cellW;
     else {
-	int xcd, col=0;
+	int xcd=0, col=0;
 	while ( col < nCols && mx > (xcd=cellWidth(col)) ) {
 	    mx -= xcd;
 	    col++;
@@ -2199,7 +2230,7 @@ int QTableView::maxRowOffset()
     if ( cellH )
 	return my/cellH;
     else {
-	int ycd, row=0;
+	int ycd=0, row=0;
 	while (my > (ycd=cellHeight(row)) ) {
 	    my -= ycd;
 	    row++;
